/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2007 The LS-Universe Team

  Homepage: http://www.ls-universe.info

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "stdafx.h"
#include "PaintHelpers.h"

#include "SettingsHelpers.h"

#include "math.h"

//For IPicture jpg and gif support
#include <olectl.h>
#include <ole2.h>

#ifndef CLEARTYPE_QUALITY
	#define CLEARTYPE_QUALITY       5
#endif

//////////////////////////////////////////////////////////////////////
// DLL Internal
//////////////////////////////////////////////////////////////////////

void RGB2HSL(unsigned char R,unsigned char G,unsigned char B,unsigned char &H,unsigned char &S,unsigned char &L)
{
	float R2, G2, B2, H2, S2, L2;

	//First scale down...
	R2 = ((float)R) / 255.0f;
	G2 = ((float)G) / 255.0f;
	B2 = ((float)B) / 255.0f;

	//...then convert with float...
	RGB2HSLFloat(R2, G2, B2, H2, S2, L2);

	//...and then rescale to 0-255.
	H = (unsigned char)(max(0, min((H2 * 255.0) / 359.0, 255)) + 0.5);
	S = (unsigned char)(max(0, min(S2 * 255.0, 255)) + 0.5);
	L = (unsigned char)(max(0, min(L2 * 255.0, 255)) + 0.5);
}

// Do a precise HSL -> RGB conversion by using
// the float functions and rescale the values
void HSL2RGB(unsigned char &R,unsigned char &G,unsigned char &B,unsigned char H,unsigned char S,unsigned char L)
{
	float R2, G2, B2, H2, S2, L2;

	//First scale down...
	H2 = (((float)H) * 359.0f) / 255.0f;
	S2 = ((float)S) / 255.0f;
	L2 = ((float)L) / 255.0f;

	//...then convert with float...
	HSL2RGBFloat(R2, G2, B2, H2, S2, L2);

	//...and then rescale to 0-255.
	R = (unsigned char)(max(0, min(R2 * 255.0, 255)) + 0.5);
	G = (unsigned char)(max(0, min(G2 * 255.0, 255)) + 0.5);
	B = (unsigned char)(max(0, min(B2 * 255.0, 255)) + 0.5);
}

//Helper-Function for color conversions
//Returns the Intensity of either R, G or B
//depending on its input
float ToRGB(float rm1, float rm2, float rh)
{
	if      (rh > 360.0f) rh -= 360.0f;
	else if (rh <   0.0f) rh += 360.0f;

	if      (rh <  60.0f) rm1 = rm1 + (rm2 - rm1) * rh / 60.0f;   
	else if (rh < 180.0f) rm1 = rm2;
	else if (rh < 240.0f) rm1 = rm1 + (rm2 - rm1) * (240.0f - rh) / 60.0f;      
	                
	return rm1;
}

//Convert a Color: RGB -> HSL
//input and output are floats, so range from RGB and HSL is not 0-255!!
//Important: Range is 0-1  but for H it is 0-360 (all with commas!)!!!!
void RGB2HSLFloat(float R, float G, float B, float &H, float &S, float &L)
{
/*#ifdef COLOR_SAFE //in safe mode evaluate the input
    R = min(max(0.0f, R), 1.0f);
	G = min(max(0.0f, G), 1.0f);
	B = min(max(0.0f, B), 1.0f);
#endif*/

	float minval, maxval, mdiff, msum;
	float rnorm, gnorm, bnorm;
    
	//Get the minimum of R, G, B
	minval = min(min(R, G), B);
	//Get the maximum of R, G, B
	maxval = max(max(R, G), B);

	mdiff  = maxval - minval;
    msum   = maxval + minval;
   
	//L(uminance) is the average of min and max
    L = msum / 2.0f;

    if (msum == 0.0f) 
    {
		//if min == max -> a grey color
		S = 0.0f;
		H = 0.0f; 
    }   
    else 
    { 
		//normalize the colors
		rnorm = (maxval - R) / mdiff;      
		gnorm = (maxval - G) / mdiff;
		bnorm = (maxval - B) / mdiff;   

		//Calculate S
		if(L <= 0.5f) S = mdiff / msum;
		else S = mdiff / (2.0f - msum);

		//Calculate H
		if (R == maxval) H = 60.0f * (6.0f + bnorm - gnorm);
		else if (G == maxval) H = 60.0f * (2.0f + rnorm - bnorm);
		else if (B == maxval) H = 60.0f * (4.0f + gnorm - rnorm);

		//do a modulo of 360 on float
		if (H > 360.0f) H = H - 360.0f;
    }
}

//Convert a Color: HSL -> RGB
//input and output are floats, so range from RGB and HSL is not 0-255!!
//Important: Range is 0-1 (with commas!) !!!!
void HSL2RGBFloat(float &R, float &G, float &B, float H, float S, float L)
{
/*#ifdef COLOR_SAFE //in safe mode evaluate the input
    H = min(max(0.0f, H), 360.0f);
	S = min(max(0.0f, S), 1.0f);
	L = min(max(0.0f, L), 1.0f);
#endif*/

	float rm1, rm2;

	//No saturation -> R, G, B is equal to L
	if(S == 0.0) R = G = B = L;
	else
	{
		if(L <= 0.5f) rm2 = L + L * S;  
		else rm2 = L + S - L * S;
		
		rm1 = 2.0f * L - rm2;
		
		//Convert H(ue) to R, G, B
		R = ToRGB(rm1, rm2, H + 120);
		G = ToRGB(rm1, rm2, H);
		B = ToRGB(rm1, rm2, H - 120);
	}
}

void PrepareIconEffects(HDC &desthDC, HBITMAP &destBitmap, COLORREF _hueColor, int _hueIntensity, int _luminanceIntensity, int _saturationIntensity, COLORREF _mixColor, int _mixIntensity, int _iconSize, HICON noAlphaMaphIcon)
{
	//Paint Saturation and Hueing Effects in MemoryDC's
	desthDC = CreateCompatibleDC(NULL);

	BITMAPINFO alpha;
	ZeroMemory( &alpha.bmiHeader, sizeof(BITMAPINFOHEADER) );
	alpha.bmiHeader.biWidth=_iconSize;      // Set size you need
	alpha.bmiHeader.biHeight=_iconSize;    // Set size you need
	alpha.bmiHeader.biPlanes=1;
	alpha.bmiHeader.biBitCount=32;      // Can be 8, 16, 32 bpp or other number
	alpha.bmiHeader.biSizeImage=0;
	alpha.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
	alpha.bmiHeader.biClrUsed=0;
	alpha.bmiHeader.biClrImportant=0;
	VOID *pvBits;

	if ( noAlphaMaphIcon != NULL && destBitmap == NULL )
	{
		destBitmap = CreateDIBSection( desthDC,
										&alpha,
										DIB_RGB_COLORS,
										&pvBits,
										NULL,
										0 );
	}

	SelectObject(desthDC, destBitmap);
	
	HDC maskDC = NULL;
	HBITMAP maskBitmap = NULL;
	HBITMAP oldmaskBitmap = NULL;

	bool useMask = false;

	if ( noAlphaMaphIcon != NULL )
	{	
		useMask = true;

		maskDC = CreateCompatibleDC(desthDC);
		maskBitmap = CreateDIBSection( maskDC,
										&alpha,
										DIB_RGB_COLORS,
										&pvBits,
										NULL,
										0 );

		oldmaskBitmap = (HBITMAP) SelectObject(maskDC, maskBitmap);

		DrawIconEx(maskDC, 0, 0, noAlphaMaphIcon, _iconSize, _iconSize, 0, NULL, DI_MASK);

		HBRUSH bgBrush = CreateSolidBrush(RGB(255,0,255));
		DrawIconEx(desthDC, 0, 0, noAlphaMaphIcon, _iconSize, _iconSize, 0, bgBrush, DI_NORMAL);
		DeleteObject(bgBrush);
	}

	BITMAP bm;
	GetObject(destBitmap, sizeof(BITMAP), &bm);
	BYTE *pPixel = (LPBYTE)bm.bmBits;

	BITMAP bm_m;
	GetObject(maskBitmap, sizeof(BITMAP), &bm_m);
	BYTE *pPixel_m = (LPBYTE)bm_m.bmBits;

	BYTE r = 0;
	BYTE g = 0;
	BYTE b = 0;

	BYTE h = 0;
	BYTE s = 0;
	BYTE l = 0;
	
	BYTE hueh = 0;
	BYTE hues = 0;
	BYTE huel = 0;

	bool needHSLConversion = false;
	bool needMixColor = false;

	if ( _luminanceIntensity != 0 || _saturationIntensity != 100 )
		needHSLConversion = true;

	if ( _hueIntensity != 0 )
	{
		r = GetRValue(_hueColor);
		g = GetGValue(_hueColor);
		b = GetBValue(_hueColor);

		if ( r == b && b == g )
		{
			_hueIntensity = 0;
		}
		else
		{
			RGB2HSL(b, g, r, hueh, hues, huel); //Switch r and b due to AlphaMap/bm_b.bmHeight * bm_b.bmWidth
			needHSLConversion = true;
		}
	}

	if ( _mixIntensity != 0 )
	{
		_mixIntensity = (int) min(255, _mixIntensity*2.55);
		needMixColor = true;
	}

	int iSize = bm.bmHeight * bm.bmWidth;

	for (int j = 0; j < iSize; ++j)
	{
		//Magic Pink and PureTransparency
		if ( (pPixel[0] == 255 && pPixel[1] == 0 && pPixel[2] == 255) || (useMask && (pPixel_m[0] == 255 && pPixel_m[1] == 255 && pPixel_m[2] == 255)) || (!useMask && pPixel[3] == 0) )
		{
			pPixel[0] = 0;
			pPixel[1] = 0;
			pPixel[2] = 0;
			pPixel[3] = 1;

			pPixel+= 4;
			if ( useMask )
				pPixel_m+= 4;
			continue;
		}

		if ( needHSLConversion || needMixColor )
		{	
			r = pPixel[0];
			g = pPixel[1];
			b = pPixel[2];

			if ( needHSLConversion )
			{
				RGB2HSL(r, g, b, h, s, l);

				if(_hueIntensity != 0)
					h = (BYTE) max(0, min(240, (h*(100-_hueIntensity)+hueh*_hueIntensity)/100));
			
				if(_luminanceIntensity != 0)
					l = (BYTE) max(0, min(240, ((l*100)+(120*_luminanceIntensity))/100));

				if(_saturationIntensity != 100)
				{
					if ( _saturationIntensity > 100 )
						s = (BYTE) max(0, min(240, (240*(_saturationIntensity-100))/100));				
					else	
						s = (BYTE) max(0, min(240, (s*_saturationIntensity)/100));
				}

				HSL2RGB(r, g, b, h, s, l);
			}

			if ( needMixColor )
			{
				// HueMix effect
				r = (BYTE)((r * (255 - _mixIntensity) + GetBValue(_mixColor) * _mixIntensity + 255) >> 8);
				g = (BYTE)((g * (255 - _mixIntensity) + GetGValue(_mixColor) * _mixIntensity + 255) >> 8);
				b = (BYTE)((b * (255 - _mixIntensity) + GetRValue(_mixColor) * _mixIntensity + 255) >> 8);
			}

			pPixel[0] = r;
			pPixel[1] = g;
			pPixel[2] = b;
		}

		if ( noAlphaMaphIcon != NULL )
		{
			pPixel[3] = 255;
		}	
		else
		{
			pPixel[0] = pPixel[0] * pPixel[3] / 255;
			pPixel[1] = pPixel[1] * pPixel[3] / 255;
			pPixel[2] = pPixel[2] * pPixel[3] / 255;
		}

		pPixel+= 4;
		if ( useMask )
			pPixel_m+= 4;
	}

	if (useMask)
	{
		maskBitmap = (HBITMAP) SelectObject(maskDC, oldmaskBitmap);
		DeleteDC(maskDC);
		DeleteObject(maskBitmap);
		DeleteObject(oldmaskBitmap);
	}
}

void PrepareImageEffects(HBITMAP &destBitmap, bool bUseAlphaMap, COLORREF _hueColor, int _hueIntensity, int _luminanceIntensity, int _saturationIntensity, COLORREF _mixColor, int _mixIntensity)
{
	if ( !destBitmap )
		return;

	//Paint Saturation and Hueing Effects in MemoryDC's
	BITMAP bm;
	GetObject(destBitmap, sizeof(BITMAP), &bm);

	BYTE *pPixel = (LPBYTE)bm.bmBits;

	BYTE r = 0;
	BYTE g = 0;
	BYTE b = 0;

	BYTE h = 0;
	BYTE s = 0;
	BYTE l = 0;
	
	BYTE hueh = 0;
	BYTE hues = 0;
	BYTE huel = 0;

	bool needHSLConversion = false;
	bool needMixColor = false;

	if ( _luminanceIntensity != 0 || _saturationIntensity != 100 )
		needHSLConversion = true;

	if ( _hueIntensity != 0 )
	{
		r = GetRValue(_hueColor);
		g = GetGValue(_hueColor);
		b = GetBValue(_hueColor);

		if ( r == b && b == g )
		{
			_hueIntensity = 0;
		}
		else
		{
			RGB2HSL(b, g, r, hueh, hues, huel); //Switch r and b due to AlphaMap/bm_b.bmHeight * bm_b.bmWidth
			needHSLConversion = true;
		}
	}

	if ( _mixIntensity != 0 )
	{
		_mixIntensity = (int) min(255, _mixIntensity*2.55);
		needMixColor = true;
	}

	int iSize = bm.bmHeight * bm.bmWidth;

	for (int i = 0; i < iSize; ++i)
	{
		//Skip Magic Pink
		if (pPixel[0] == 255 && pPixel[1] == 0 && pPixel[2] == 255)
		{
			pPixel[0] = 255;
			pPixel[1] = 0;
			pPixel[2] = 255;
			pPixel[3] = 255;
			
			pPixel+= 4;
			continue;
		}
		//Prepare Text or !bUseAlphaMap
		else if (pPixel[3] == 0 && bUseAlphaMap)
			pPixel[3] = 1;
		//Prepare for NonAlphaMap
		else if (pPixel[3] == 0 && !bUseAlphaMap)
			pPixel[3] = 255;

		r = pPixel[0];
		g = pPixel[1];
		b = pPixel[2];

		if ( needHSLConversion )
		{
			//AlphaMap Colorizing
			RGB2HSL(r, g, b, h, s, l);

			if(_hueIntensity != 0)
				h = (BYTE) max(0, min(240, (h*(100-_hueIntensity)+hueh*_hueIntensity)/100));
		
			if(_luminanceIntensity != 0)
				l = (BYTE) max(0, min(240, ((l*100)+(120*_luminanceIntensity))/100));

			if(_saturationIntensity != 100)
			{
				if ( _saturationIntensity > 100 )
					s = (BYTE) max(0, min(240, (240*(_saturationIntensity-100))/100));				
				else
					s = (BYTE) max(0, min(240, (s*_saturationIntensity)/100));
			}

			HSL2RGB(r, g, b, h, s, l);
		}

		if ( needMixColor )
		{
			// HueMix effect
			r = (BYTE)((r * (255 - _mixIntensity) + GetBValue(_mixColor) * _mixIntensity + 255) >> 8);
			g = (BYTE)((g * (255 - _mixIntensity) + GetGValue(_mixColor) * _mixIntensity + 255) >> 8);
			b = (BYTE)((b * (255 - _mixIntensity) + GetRValue(_mixColor) * _mixIntensity + 255) >> 8);
		}
		
		//PreMultiply
		if ( pPixel[3] == 1 )
		{
			pPixel[0] = 0;
			pPixel[1] = 0;
			pPixel[2] = 0;
		}
		else
		{
			pPixel[0] = r * pPixel[3] / 255;
			pPixel[1] = g * pPixel[3] / 255;
			pPixel[2] = b * pPixel[3] / 255;
		}

		pPixel+= 4;
	}
}

void DrawMultiGradient(HDC desthDC, RECT destRect, ColorRefList GradientColors, ColorRepeatedList GradientRepeatedColors, bool _verticalGradient)
{
	if ( GradientRepeatedColors.empty() )
	{
		int nWidth = destRect.right-destRect.left;
		int nHeight = destRect.bottom-destRect.top;

		RECT r2 = {0,0,0,0};
		double cr, cg, cb, dr, dg, db;

		// ColorsSteps between Start and End Color
		int gradSteps = 0;
		int unequalRest = 0;

		int colorCount = GradientColors.size();

		if (_verticalGradient)
		{
			gradSteps = (nHeight-colorCount)/(colorCount-1);
			unequalRest = (nHeight-colorCount)%(colorCount-1);
		}
		else
		{
			gradSteps = (nWidth-colorCount)/(colorCount-1);
			unequalRest = (nWidth-colorCount)%(colorCount-1);
		}

		int lastGradientPos = 0;

		if (_verticalGradient)
			lastGradientPos = destRect.top;
		else
			lastGradientPos = destRect.left;

		int counter = 0;
		for (ColorRefListIterator cit = GradientColors.begin(); cit != GradientColors.end(); cit++ )
		{
			if (cit == GradientColors.end())
				break;

			counter++;

			COLORREF one = *cit;
			cit++;
			COLORREF two = *cit;
			cit--;
		
			// get current color
			cr = GetRValue( one );
			cg = GetGValue( one );
			cb = GetBValue( one );

			// Get delta color (+1 cause we must reach the Final Color)
			dr = (cr - GetRValue( two )) / (gradSteps+1);
			dg = (cg - GetGValue( two )) / (gradSteps+1);
			db = (cb - GetBValue( two )) / (gradSteps+1);

			//Paint Real Gradient Color
			//-------------------------------------------
			if ( unequalRest != 0 && counter > 1 )
			{
				unequalRest--;

				if (_verticalGradient)
					SetRect(&r2, destRect.left, lastGradientPos, destRect.right, min(lastGradientPos+1, destRect.bottom));
				else
					SetRect(&r2, lastGradientPos, destRect.top, min(lastGradientPos+1, destRect.right), destRect.bottom);

				if (r2.right-r2.left > 0 && r2.bottom-r2.top > 0)
				{
					::SetBkColor(desthDC, one);
					::ExtTextOut(desthDC, 0, 0, ETO_OPAQUE, &r2, NULL, 0, NULL);
				}
				
				if (_verticalGradient)
					lastGradientPos = r2.bottom;
				else
					lastGradientPos = r2.right;
			}
			//-------------------------------------------

			for (int i = 0; i <= gradSteps; i++)
			{
				if (_verticalGradient)
					SetRect(&r2, destRect.left, lastGradientPos+i, destRect.right, min(lastGradientPos+i+1, destRect.bottom));
				else
					SetRect(&r2, lastGradientPos+i, destRect.top, min(lastGradientPos+i+1, destRect.right), destRect.bottom);

				if (r2.right-r2.left > 0 && r2.bottom-r2.top > 0)
				{
					::SetBkColor(desthDC, RGB((int)cr, (int)cg, (int)cb));
					::ExtTextOut(desthDC, 0, 0, ETO_OPAQUE, &r2, NULL, 0, NULL);
				}
				
				cr -= dr;
				cg -= dg;
				cb -= db;
			}

			if (_verticalGradient)
				lastGradientPos = r2.bottom;
			else
				lastGradientPos = r2.right;
		}
	}
	else
	{
		int nWidth = destRect.right-destRect.left;
		int nHeight = destRect.bottom-destRect.top;

		RECT r2 = {0,0,0,0};
		COLORREF color;

		int lastGradientPos = 0;

		if (_verticalGradient)
			lastGradientPos = destRect.top;
		else
			lastGradientPos = destRect.left;

		int counter = 0;
		int nSize = 0;
		
		if (_verticalGradient)
			nSize = nHeight;
		else
			nSize = nWidth; 

		ColorRepeatedListIterator iit = GradientRepeatedColors.begin();
		int _repeatedGradient = (*iit);
		
		while ( counter < nSize )
		{
			for (ColorRefListIterator cit = GradientColors.begin(); cit != GradientColors.end(); cit++ )
			{
				color = *cit;

				if (_verticalGradient)
					SetRect(&r2, destRect.left, lastGradientPos, destRect.right, min(lastGradientPos+_repeatedGradient, destRect.bottom));
				else
					SetRect(&r2, lastGradientPos, destRect.top, min(lastGradientPos+_repeatedGradient, destRect.right), destRect.bottom);

				if (r2.right-r2.left > 0 && r2.bottom-r2.top > 0)
				{
					::SetBkColor(desthDC, color);
					::ExtTextOut(desthDC, 0, 0, ETO_OPAQUE, &r2, NULL, 0, NULL);
				}

				if (_verticalGradient)
					lastGradientPos = r2.bottom;
				else
					lastGradientPos = r2.right;

				counter++;

				iit++;
				if ( iit == GradientRepeatedColors.end() )
					iit = GradientRepeatedColors.begin();
				_repeatedGradient = (*iit);

				if ( counter == nSize )
					break;
			}
		}
	}
}

void DrawSpecialGradient(HDC desthDC, RECT destRect, COLORREF _startColor, COLORREF _endColor, int _gradientType, int _transformationType)
{
	int i, j, y, x1, y1;
	double r, d, n1;

	// Calculate image size
	int width = destRect.right - destRect.left;
	int height = destRect.bottom - destRect.top;

	int size = height*width*4;
	double radius = sqrt( pow(height/2,2) + pow(width/2,2) );
	double k = double(destRect.bottom-destRect.top)/double(destRect.right-destRect.left);
	if ( _gradientType == GRADIENT_BDIAGONAL )
		k = double(destRect.bottom-destRect.top)/double(destRect.left-destRect.right);
	if ( ( _gradientType == GRADIENT_FDIAGONAL ) || ( _gradientType == GRADIENT_BDIAGONAL ) )
		_gradientType = GRADIENT_DIAGONAL;
	double n = 0.0;
	double k1 = -1/k;

	// Create data buffer
	unsigned char* m_Data = new unsigned char[size];
	
	// Create gradient
	for ( i=0; i<height; i++ )
	{
		for ( j=0; j<width; j++ )
		{
			switch (_gradientType)
			{
				// Horizontal gradient
				case GRADIENT_HORIZONTAL:
				{
					m_Data[i*width*4 + j*4] = (unsigned char)(((double)j/(double)width)*GetBValue(_endColor) + (1-(double)j/(double)width)*GetBValue(_startColor));
					m_Data[i*width*4 + j*4+1] = (unsigned char)(((double)j/(double)width)*GetGValue(_endColor) + (1-(double)j/(double)width)*GetGValue(_startColor));
					m_Data[i*width*4 + j*4+2] = (unsigned char)(((double)j/(double)width)*GetRValue(_endColor) + (1-(double)j/(double)width)*GetRValue(_startColor));
				}
				break;

				// Vertical gradient
				case GRADIENT_VERTICAL:
				{
					m_Data[i*width*4 + j*4] = (unsigned char)(((double)i/(double)height)*GetBValue(_endColor) + (1-(double)i/(double)height)*GetBValue(_startColor));
					m_Data[i*width*4 + j*4+1] = (unsigned char)(((double)i/(double)height)*GetGValue(_endColor) + (1-(double)i/(double)height)*GetGValue(_startColor));
					m_Data[i*width*4 + j*4+2] = (unsigned char)(((double)i/(double)height)*GetRValue(_endColor) + (1-(double)i/(double)height)*GetRValue(_startColor));
				}
				break;

				// Radial gradient
				case GRADIENT_RADIAL:
				{
					r = sqrt( pow(double(i-height/2)/radius,2) + pow(double(j-width/2)/radius,2) );
					if ( r <= 1.0 )
					{
						m_Data[i*width*4 + j*4] = (unsigned char)(r*GetBValue(_endColor) + (1-r)*GetBValue(_startColor));
						m_Data[i*width*4 + j*4+1] = (unsigned char)(r*GetGValue(_endColor) + (1-r)*GetGValue(_startColor));
						m_Data[i*width*4 + j*4+2] = (unsigned char)(r*GetRValue(_endColor) + (1-r)*GetRValue(_startColor));
					}
					else
					{
						m_Data[i*width*4 + j*4] = GetBValue(_endColor);
						m_Data[i*width*4 + j*4+1] = GetGValue(_endColor);
						m_Data[i*width*4 + j*4+2] = GetRValue(_endColor);
					}
				}
				break;

				// Diagonal gradient
				case GRADIENT_DIAGONAL:
				{
					y = int(k*(j-width/2) + n);
					n1 = int((i-height/2) - k1*(j-width/2));
					x1 = int((n1-n)/(k-k1));
					y1 = int(k1*x1 + n1);
					d = sqrt( pow((j-width/2)-x1,2) + pow((i-height/2)-y1,2) ) / (2*radius);
					if ( (i-height/2) > y )
					{
						m_Data[i*width*4 + j*4] = (unsigned char)((0.5-d)*GetBValue(_endColor) + (0.5+d)*GetBValue(_startColor));
						m_Data[i*width*4 + j*4+1] = (unsigned char)((0.5-d)*GetGValue(_endColor) + (0.5+d)*GetGValue(_startColor));
						m_Data[i*width*4 + j*4+2] = (unsigned char)((0.5-d)*GetRValue(_endColor) + (0.5+d)*GetRValue(_startColor));
					}
					else
					{
						m_Data[i*width*4 + j*4] = (unsigned char)((0.5-d)*GetBValue(_startColor) + (0.5+d)*GetBValue(_endColor));
						m_Data[i*width*4 + j*4+1] = (unsigned char)((0.5-d)*GetGValue(_startColor) + (0.5+d)*GetGValue(_endColor));
						m_Data[i*width*4 + j*4+2] = (unsigned char)((0.5-d)*GetRValue(_startColor) + (0.5+d)*GetRValue(_endColor));
					}
				}
				break;
			}
		}
	}

	if ( _transformationType != TRANSFORMATION_NONE )
		ApplyTransformation( m_Data, width, height, _transformationType );

	// Create temporary bitmap
	HBITMAP hBitmap = (HBITMAP)CreateBitmap( width, height, 1, 32, m_Data );

	// Create temporary device context
	HDC hMemDC = CreateCompatibleDC(desthDC);
	HBITMAP hOldBitmap = (HBITMAP)SelectObject( hMemDC, hBitmap );

	// Blit gradient bitmap
	BitBlt( desthDC, destRect.left, destRect.top, width, height, hMemDC, 0, 0, SRCCOPY );

	// Free memory
	hBitmap = (HBITMAP) SelectObject( hMemDC, hOldBitmap );
	DeleteDC(hMemDC);
	DeleteObject(hBitmap);
	DeleteObject(hOldBitmap);
	delete m_Data;
	m_Data = NULL;
}

void ApplyTransformation(unsigned char* m_Data, int width, int height, int _transformationType)
{
	int X, Y;
	double r, fi;
	double R = sqrt( pow(width/4,2) + pow(height/4,2) );
	double FI = atan( double(height)/double(width) );

	int size = height*width*4;
	unsigned char* newBuffer = new unsigned char[size];
	memcpy( newBuffer, m_Data, size );

	for ( int i=0; i<height; i++ )
	{
		for ( int j=0; j<width; j++ )
		{
			switch (_transformationType)
			{
				case TRANSFORMATION_CHARICATURE:
				{
					fi = atan(double(i-height/2)/double(j-width/2));
					r = sqrt( pow((i-height/2),2) + pow((j-width/2),2) );
					X = int(sqrt(r*R*2)*cos(fi));
					Y = int(fabs(sqrt(r*R*2)*sin(fi)));
					newBuffer[i*width*4 + j*4] = m_Data[Y*width*4 + X*4];
					newBuffer[i*width*4 + j*4+1] = m_Data[Y*width*4 + X*4+1];
					newBuffer[i*width*4 + j*4+2] = m_Data[Y*width*4 + X*4+2];
				}
				break;

				case TRANSFORMATION_FISHEYE:
				{
					fi = atan(double(i-height/2)/double(j-width/2));
					r = sqrt( pow((i-height/2),2) + pow((j-width/2),2) );
					X = int((pow(r,2)/R)*cos(fi));
					Y = int(fabs((pow(r,2)/R)*sin(fi))) % height;
					newBuffer[i*width*4 + j*4] = m_Data[Y*width*4 + X*4];
					newBuffer[i*width*4 + j*4+1] = m_Data[Y*width*4 + X*4+1];
					newBuffer[i*width*4 + j*4+2] = m_Data[Y*width*4 + X*4+2];
				}
				break;

				case TRANSFORMATION_SWIRLED:
				{
					fi = atan(double(i-height/2)/double(j-width/2));
					r = sqrt( pow((i-height/2),2) + pow((j-width/2),2) );
					X = int(fabs(r*cos(fi+r/8)));
					Y = int(fabs(r*sin(fi+r/8)));
					newBuffer[i*width*4 + j*4] = m_Data[Y*width*4 + X*4];
					newBuffer[i*width*4 + j*4+1] = m_Data[Y*width*4 + X*4+1];
					newBuffer[i*width*4 + j*4+2] = m_Data[Y*width*4 + X*4+2];
				}
				break;

				case TRANSFORMATION_CYLINDER:
				{
					fi = atan(double(i-height/2)/double(j-width/2));
					r = sqrt( pow((i-height/2),2) + pow((j-width/2),2) );
					X = abs((j-width/2) + int(fi+r/10)%32 - 8);
					Y = i;
					newBuffer[i*width*4 + j*4] = m_Data[Y*width*4 + X*4];
					newBuffer[i*width*4 + j*4+1] = m_Data[Y*width*4 + X*4+1];
					newBuffer[i*width*4 + j*4+2] = m_Data[Y*width*4 + X*4+2];
				}
				break;

				case TRANSFORMATION_SHIFT:
				{
					fi = atan(double(i-height/2)/double(j-width/2));
					r = sqrt( pow((i-height/2),2) + pow((j-width/2),2) );
					X = int(fabs(r*cos(fi)));
					Y = int(fabs(r*sin(fi)));
					newBuffer[i*width*4 + j*4] = m_Data[Y*width*4 + X*4];
					newBuffer[i*width*4 + j*4+1] = m_Data[Y*width*4 + X*4+1];
					newBuffer[i*width*4 + j*4+2] = m_Data[Y*width*4 + X*4+2];
				}
				break;
			}
		}
	}

	memcpy( m_Data, newBuffer, size );
	delete newBuffer;
}

void CreateFontHandle(HFONT &Font, string strFontName, int iFontHeight, bool bFontItalic, bool bFontUnderline, bool bFontBold, short shFontCharSet, bool bFontSmoothing, bool bFontClearType)
{
	LOGFONT lf;
	memset(&lf, 0, sizeof(LOGFONT));

	StringCchCopy(lf.lfFaceName, LF_FACESIZE, strFontName.c_str());
	lf.lfHeight = iFontHeight;
	lf.lfItalic = bFontItalic ? TRUE : FALSE;
	lf.lfUnderline = bFontUnderline ? TRUE : FALSE;
	lf.lfWeight = bFontBold ? FW_BOLD : FW_NORMAL;
    lf.lfCharSet = shFontCharSet;
	if ( IsOS( OS_XP ) && bFontSmoothing && bFontClearType )
		lf.lfQuality = CLEARTYPE_QUALITY;
	else if ( bFontSmoothing )
		lf.lfQuality = ANTIALIASED_QUALITY;
	else
		lf.lfQuality = NONANTIALIASED_QUALITY;

	if ( Font )
	{
		DeleteObject(Font);
		Font = NULL;
	}

	Font = CreateFontIndirect(&lf);
}

POINT MeasureTextSize(HDC desthDC, HFONT &Font, const string &measureText)
{
	POINT ptNeededSize;

	HFONT tmpFont = (HFONT) SelectObject(desthDC, Font);

	LPSIZE lpsize = new SIZE();
	lpsize->cx = 100;
	lpsize->cy = 100;

	GetTextExtentPoint32(desthDC, measureText.c_str(), strlen(measureText.c_str()), lpsize);

	Font = (HFONT) SelectObject(desthDC, tmpFont);

	DeleteObject(tmpFont);

	ptNeededSize.x = lpsize->cx;
	ptNeededSize.y = lpsize->cy;

    delete lpsize;

	return ptNeededSize;
}

//Return values: 1 PNG, 2 BMP, 3 JPG, 4 GIF
int GetFileType(string strFileName)
{
	CStr strFileNameEx = strFileName.c_str();
	strFileNameEx.Lower();

	if ( strFileNameEx.ReverseFind(".png") == strFileNameEx.Length()-4 )
		return 1;
	else if ( strFileNameEx.ReverseFind(".bmp") == strFileNameEx.Length()-4 )
		return 2;
	else if ( strFileNameEx.ReverseFind(".jpg") == strFileNameEx.Length()-4 || strFileNameEx.ReverseFind(".jpeg") == strFileNameEx.Length()-5 )
		return 3;
	else if ( strFileNameEx.ReverseFind(".gif") == strFileNameEx.Length()-4 )
		return 4;
	else
		return 2; //Uses LS Default processing for BuiltIn LS features
}

HBITMAP LoadLSImageEx(string strFileName)
{
	HBITMAP resultBitmap = NULL; 

	CStr strFileNameEx = strFileName.c_str();
	strFileNameEx.Lower();

	int iFileType = GetFileType(strFileName);

	if ( iFileType >= 3 )
	{
		//Check first $LSImageFolder
		char szImageFolder[MAX_PATH] = {0};
	
		VarExpansionEx(szImageFolder, "$lsimagefolder$", MAX_PATH);

		StringCchCat(szImageFolder, MAX_PATH, strFileName.c_str());

		resultBitmap = LoadJPGGIFImage((string) szImageFolder, iFileType);

		//Then try a full path
		if ( !resultBitmap )
			resultBitmap = LoadJPGGIFImage(strFileName, iFileType);

		if ( resultBitmap )
			return resultBitmap;
	}

	resultBitmap = LoadLSImage(strFileName.c_str(), 0);

	return resultBitmap;
}

HBITMAP LoadJPGGIFImage(string strFileName, int iFileType)
{
	// Use IPicture stuff to use JPG / GIF files
	// Read file in memory
	FILE *fp = fopen(strFileName.c_str(), "rb");
	if (!fp)
		return NULL;

	fseek(fp,0,SEEK_END);
	int fs = ftell(fp);
	fseek(fp,0,SEEK_SET);
	HGLOBAL hG = GlobalAlloc(GPTR,fs);
	if (!hG)
	{
		fclose(fp);
		return NULL;
	}

	void* pp = (void*) hG;
	fread(pp,1,fs,fp);
	fclose(fp);

	// Create an IStream so IPicture can
	// CreateStreamOnHGlobal(hG,false,&s);
	IStream* s = NULL;
	CreateStreamOnHGlobal(hG, false, &s);
	if (!s)
	{
		GlobalFree(hG);
		return NULL;
	}

	IPicture* p = NULL;
	OleLoadPicture(s, 0, false, IID_IPicture, (void**) &p);

	if (!p)
	{
		s->Release();
		GlobalFree(hG);
		return NULL;
	}

	s->Release();
	GlobalFree(hG);

	HBITMAP resultBitmap = NULL;

	if ( iFileType == 3 )
	{
		HBITMAP hB = NULL;

		p->get_Handle((unsigned int*) &hB);

		// Copy the image. Necessary, because upon p's release,
		// the handle is destroyed.
		resultBitmap = (HBITMAP) CopyImage(hB, IMAGE_BITMAP, 0, 0, LR_COPYRETURNORG | LR_CREATEDIBSECTION);

		p->Release();
	}
	else
	{	
		OLE_XSIZE_HIMETRIC width;
		OLE_YSIZE_HIMETRIC height;
		p->get_Width( &width ) ;
		p->get_Height( &height ) ;

		//Convert HIMETRIC to Pixel
		int iWidth = width*96/2540;
		int iHeight = height*96/2540;

		HDC bufferDC = CreateCompatibleDC(NULL);

		BITMAPINFO alpha;
		ZeroMemory( &alpha.bmiHeader, sizeof(BITMAPINFOHEADER) );
		alpha.bmiHeader.biWidth=iWidth;      // Set size you need
		alpha.bmiHeader.biHeight=iHeight;    // Set size you need
		alpha.bmiHeader.biPlanes=1;
		alpha.bmiHeader.biBitCount=32;      // Can be 8, 16, 32 bpp or other number
		alpha.bmiHeader.biSizeImage=0;
		alpha.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
		alpha.bmiHeader.biClrUsed= 0;
		alpha.bmiHeader.biClrImportant= 0;
		VOID *pvBits;

		HBITMAP bufferBitmap = CreateDIBSection( bufferDC,
													&alpha,
													DIB_RGB_COLORS,
													&pvBits,
													NULL,
													0 );

		HBITMAP oldBMP = (HBITMAP) SelectObject(bufferDC, bufferBitmap);

		//Set Image to Magic Pink
		BITMAP bm;
		GetObject(bufferBitmap, sizeof(BITMAP), &bm);
		BYTE *pPixel = (LPBYTE)bm.bmBits;

		int iSize = bm.bmHeight * bm.bmWidth;

		for (int i = 0; i < iSize; ++i)
		{
			pPixel[0] = 255;
			pPixel[1] = 0;
			pPixel[2] = 255;
			pPixel[3] = 255;
		
			pPixel+= 4;
		}

		//Render (Transparent) Gif
		p->Render( bufferDC,
                    0,
                    0,
                    iWidth, iHeight,           
                    0,
                    height,
                    width,
                    -height,
                    NULL ) ;

		bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);

		resultBitmap = (HBITMAP) CopyImage(bufferBitmap, IMAGE_BITMAP, 0, 0, LR_COPYRETURNORG | LR_CREATEDIBSECTION);

		DeleteDC(bufferDC);	
		DeleteObject(bufferBitmap);
		DeleteObject(oldBMP);

		p->Release();
	}

	return resultBitmap;
}

//////////////////////////////////////////////////////////////////////
//Exported in xExports.h
//////////////////////////////////////////////////////////////////////

// tile an image horizontally and vertically
void TileBlt(HDC hdcDest, int xDest, int yDest, int cxDest, int cyDest, HDC hdcSrc, int xSrc, int ySrc, int cxSrc, int cySrc, DWORD rasterOp)
{
	for(int y = 0; y < cyDest; y += cySrc)
	{
		for(int x = 0; x < cxDest; x += cxSrc)
		{
			BitBlt(hdcDest,
				xDest + x,
				yDest + y,
				min(cxDest - x, cxSrc),
				min(cyDest - y, cySrc),
				hdcSrc,
				xSrc,
				ySrc,
				rasterOp);
		}
	}
}

// tile an image horizontally
void HorizontalTileBlt(HDC hdcDest, int xDest, int yDest, int cxDest, int cyDest, HDC hdcSrc, int xSrc, int ySrc, int cxSrc, int cySrc, DWORD rasterOp)
{
	//SetStretchBltMode(hdcDest, STRETCH_HALFTONE);

	for(int x = 0; x < cxDest; x += cxSrc)
	{
		StretchBlt(hdcDest,
			xDest + x,
			yDest,
			min(cxDest - x, cxSrc),
			cyDest,
			hdcSrc,
			xSrc,
			ySrc,
			min(cxDest - x, cxSrc),
			cySrc,
			rasterOp);
	}
}

// tile an image vertically
void VerticalTileBlt(HDC hdcDest, int xDest, int yDest, int cxDest, int cyDest, HDC hdcSrc, int xSrc, int ySrc, int cxSrc, int cySrc, DWORD rasterOp)
{
	//SetStretchBltMode(hdcDest, STRETCH_HALFTONE);

	for(int y = 0; y < cyDest; y += cySrc)
	{
		StretchBlt(hdcDest,
			xDest,
			yDest + y,
			cxDest,
			min(cyDest - y, cySrc),
			hdcSrc,
			xSrc,
			ySrc,
			cxSrc,
			min(cyDest - y, cySrc),
			rasterOp);
	}
}

// multi-blt!
void MultiBlt(HDC hdcDest, int xDest, int yDest, int cxDest, int cyDest, HDC hdcSrc, int xSrc, int ySrc, int cxSrc, int cySrc, int cxLeft, int cyTop, int cxRight, int cyBottom, int mode, DWORD rasterOp)
{
	//SetStretchBltMode(hdcDest, STRETCH_HALFTONE);

	if(cyTop > 0)
	{
		if(cxLeft > 0)
		{
			BitBlt(hdcDest,
				xDest,
				yDest,
				cxLeft,
				cyTop,
				hdcSrc,
				xSrc,
				ySrc,
				rasterOp);
		}

		if(mode == MULTIBLT_STRETCH || mode == MULTIBLT_TILEVERTICAL)
		{
			StretchBlt(hdcDest,
				xDest + cxLeft,
				yDest,
				cxDest - cxLeft - cxRight,
				cyTop,
				hdcSrc,
				xSrc + cxLeft,
				ySrc,
				cxSrc - cxLeft - cxRight,
				cyTop,
				rasterOp);
		}
		else
		{
			HorizontalTileBlt(hdcDest,
				xDest + cxLeft,
				yDest,
				cxDest - cxLeft - cxRight,
				cyTop,
				hdcSrc,
				xSrc + cxLeft,
				ySrc,
				cxSrc - cxLeft - cxRight,
				cyTop,
				rasterOp);
		}

		if(cxRight > 0)
		{
			BitBlt(hdcDest,
				cxDest - cxRight,
				yDest,
				cxRight,
				cyTop,
				hdcSrc,
				cxSrc - cxRight,
				ySrc,
				rasterOp);
		}
	}

	if(cxLeft > 0)
	{
		if(mode == MULTIBLT_STRETCH || mode == MULTIBLT_TILEHORIZONTAL)
		{
			StretchBlt(hdcDest,
				xDest,
				yDest + cyTop,
				cxLeft,
				cyDest - cyTop - cyBottom,
				hdcSrc,
				xSrc,
				ySrc + cyTop,
				cxLeft,
				cySrc - cyTop - cyBottom,
				rasterOp);
		}
		else
		{
			VerticalTileBlt(hdcDest,
				xDest,
				yDest + cyTop,
				cxLeft,
				cyDest - cyTop - cyBottom,
				hdcSrc,
				xSrc,
				ySrc + cyTop,
				cxLeft,
				cySrc - cyTop - cyBottom,
				rasterOp);
		}
	}

	if(mode == MULTIBLT_STRETCH)
	{
		StretchBlt(hdcDest,
			xDest + cxLeft,
			yDest + cyTop,
			cxDest - cxLeft - cxRight,
			cyDest - cyTop - cyBottom,
			hdcSrc,
			xSrc + cxLeft,
			ySrc + cyTop,
			cxSrc - cxLeft - cxRight,
			cySrc - cyTop - cyBottom,
			rasterOp);
	}
	else if(mode == MULTIBLT_TILE)
	{
		TileBlt(hdcDest,
			xDest + cxLeft,
			yDest + cyTop,
			cxDest - cxLeft - cxRight,
			cyDest - cyTop - cyBottom,
			hdcSrc,
			xSrc + cxLeft,
			ySrc + cyTop,
			cxSrc - cxLeft - cxRight,
			cySrc - cyTop - cyBottom,
			rasterOp);
	}
	else if(mode == MULTIBLT_TILEHORIZONTAL)
	{
		HorizontalTileBlt(hdcDest,
			xDest + cxLeft,
			yDest + cyTop,
			cxDest - cxLeft - cxRight,
			cyDest - cyTop - cyBottom,
			hdcSrc,
			xSrc + cxLeft,
			ySrc + cyTop,
			cxSrc - cxLeft - cxRight,
			cySrc - cyTop - cyBottom,
			rasterOp);
	}
	else if(mode == MULTIBLT_TILEVERTICAL)
	{
		VerticalTileBlt(hdcDest,
			xDest + cxLeft,
			yDest + cyTop,
			cxDest - cxLeft - cxRight,
			cyDest - cyTop - cyBottom,
			hdcSrc,
			xSrc + cxLeft,
			ySrc + cyTop,
			cxSrc - cxLeft - cxRight,
			cySrc - cyTop - cyBottom,
			rasterOp);
	}

	if(cxRight > 0)
	{
		if(mode == MULTIBLT_STRETCH || mode == MULTIBLT_TILEHORIZONTAL)
		{
			StretchBlt(hdcDest,
				cxDest - cxRight,
				yDest + cyTop,
				cxRight,
				cyDest - cyTop - cyBottom,
				hdcSrc,
				cxSrc - cxRight,
				ySrc + cyTop,
				cxRight,
				cySrc - cyTop - cyBottom,
				rasterOp);
		}
		else
		{
			VerticalTileBlt(hdcDest,
				cxDest - cxRight,
				yDest + cyTop,
				cxRight,
				cyDest - cyTop - cyBottom,
				hdcSrc,
				cxSrc - cxRight,
				ySrc + cyTop,
				cxRight,
				cySrc - cyTop - cyBottom,
				rasterOp);
		}
	}

	if(cyBottom > 0)
	{
		if(cxLeft > 0)
		{
			BitBlt(hdcDest,
				xDest,
				cyDest - cyBottom,
				cxLeft,
				cyBottom,
				hdcSrc,
				xSrc,
				cySrc - cyBottom,
				rasterOp);
		}

		if(mode == MULTIBLT_STRETCH || mode == MULTIBLT_TILEVERTICAL)
		{
			StretchBlt(hdcDest,
				xDest + cxLeft,
				cyDest - cyBottom,
				cxDest - cxLeft - cxRight,
				cyBottom,
				hdcSrc,
				xSrc + cxLeft,
				cySrc - cyBottom,
				cxSrc - cxLeft - cxRight,
				cyBottom,
				rasterOp);
		}
		else
		{
			HorizontalTileBlt(hdcDest,
				xDest + cxLeft,
				cyDest - cyBottom,
				cxDest - cxLeft - cxRight,
				cyBottom,
				hdcSrc,
				xSrc + cxLeft,
				cySrc - cyBottom,
				cxSrc - cxLeft - cxRight,
				cyBottom,
				rasterOp);
		}

		if(cxRight > 0)
		{
			BitBlt(hdcDest,
				cxDest - cxRight,
				cyDest - cyBottom,
				cxRight,
				cyBottom,
				hdcSrc,
				cxSrc - cxRight,
				cySrc - cyBottom,
				rasterOp);
		}
	}
}
