/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2007 The LS-Universe Team

  Homepage: http://www.ls-universe.info

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "stdafx.h"
#include "PaintText.h"

#include "SettingsHelpers.h"
#include "PaintHelpers.h"

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

PaintText::PaintText() :
	m_bUseAlphaMap(false),

	m_bFontFade(false),

	Font(NULL),
	tmpFont(NULL),

	m_bFontVisible(true),

	m_iFontAlphaTransparency(255),

	m_strBorders(""),
	m_iLeftBorder(0),
	m_iTopBorder(0),
	m_iRightBorder(0),
	m_iBottomBorder(0),

	m_strFontName("Arial"),
	m_iFontHeight(15),
	m_clrFontColor(RGB(0,0,0)),

	m_shFontCharSet(DEFAULT_CHARSET),

	m_bFontSmoothing(true),
	m_bFontClearType(true),

	m_bFontBold(false),
	m_bFontItalic(false),
	m_bFontUnderline(false),

	m_bFontShadow(false),
	m_iFontShadowX(1),
	m_iFontShadowY(1),
	m_iFontBlockedShadow(0),
	m_clrFontShadowColor(RGB(128,128,128)),

	m_bFontOutline(false),
	m_clrFontOutlineColor(RGB(128,128,128)),
	m_bFontOutlineExtra(false),

	m_clrFontEmbossColor1(RGB(255,0,255)),
	m_clrFontEmbossColor2(RGB(255,0,255)),

	m_iFontAlign(DT_LEFT),
	m_iFontVertAlign(DT_VCENTER),

	m_bFontNoEllipsis(false),

	m_strFontFadeCFG(""),
	m_iLeftFade(0),
	m_iRightFade(0),
	m_iTopFade(0),
	m_iBottomFade(0),

	//Update 4
	m_iFontRotation(0),
	m_iRotationLeftBorder(0),
	m_iRotationTopBorder(0),
	m_iRotationRightBorder(0),
	m_iRotationBottomBorder(0)
{
}

PaintText::PaintText(const PaintTextDefaults *pPaintTextDefaults) :
	m_bUseAlphaMap(false),

	m_bFontFade(false),

	Font(NULL),
	tmpFont(NULL)
{
	m_bFontVisible = (pPaintTextDefaults->m_bFontVisible ? true : false);

	m_iFontAlphaTransparency = pPaintTextDefaults->m_iFontAlphaTransparency;

	m_strBorders = pPaintTextDefaults->m_strBorders;
	m_iLeftBorder = pPaintTextDefaults->m_iLeftBorder;
	m_iTopBorder = pPaintTextDefaults->m_iTopBorder;
	m_iRightBorder = pPaintTextDefaults->m_iRightBorder;
	m_iBottomBorder = pPaintTextDefaults->m_iBottomBorder;

	m_strFontName = pPaintTextDefaults->m_strFontName;
	m_iFontHeight = pPaintTextDefaults->m_iFontHeight;
	m_clrFontColor = pPaintTextDefaults->m_clrFontColor;

	m_shFontCharSet = pPaintTextDefaults->m_shFontCharSet;

	m_bFontSmoothing = (pPaintTextDefaults->m_bFontSmoothing ? true : false);
	m_bFontClearType = (pPaintTextDefaults->m_bFontClearType ? true : false);

	m_bFontBold = (pPaintTextDefaults->m_bFontBold ? true : false);
	m_bFontItalic = (pPaintTextDefaults->m_bFontItalic ? true : false);
	m_bFontUnderline = (pPaintTextDefaults->m_bFontUnderline ? true : false);

	m_bFontShadow = (pPaintTextDefaults->m_bFontShadow ? true : false);
	m_iFontShadowX = pPaintTextDefaults->m_iFontShadowX;
	m_iFontShadowY = pPaintTextDefaults->m_iFontShadowY;
	m_iFontBlockedShadow = pPaintTextDefaults->m_iFontBlockedShadow;
	m_clrFontShadowColor = pPaintTextDefaults->m_clrFontShadowColor;

	m_bFontOutline = (pPaintTextDefaults->m_bFontOutline ? true : false);
	m_clrFontOutlineColor = pPaintTextDefaults->m_clrFontOutlineColor;
	m_bFontOutlineExtra = (pPaintTextDefaults->m_bFontOutlineExtra ? true : false);

	m_clrFontEmbossColor1 = pPaintTextDefaults->m_clrFontEmbossColor1;
	m_clrFontEmbossColor2 = pPaintTextDefaults->m_clrFontEmbossColor2;

	m_iFontAlign = pPaintTextDefaults->m_iFontAlign;
	m_iFontVertAlign = pPaintTextDefaults->m_iFontVertAlign;

	m_bFontNoEllipsis = (pPaintTextDefaults->m_bFontNoEllipsis ? true : false);

	m_strFontFadeCFG = pPaintTextDefaults->m_strFontFadeCFG;
	m_iLeftFade = pPaintTextDefaults->m_iLeftFade;
	m_iRightFade = pPaintTextDefaults->m_iRightFade;
	m_iTopFade = pPaintTextDefaults->m_iTopFade;
	m_iBottomFade = pPaintTextDefaults->m_iBottomFade;

	//Update 4
	//Cannot enable due to backwards compatibility!
	m_iFontRotation = 0;//pPaintTextDefaults->m_iFontRotation;
	m_iRotationLeftBorder = 0;
	m_iRotationTopBorder = 0;
	m_iRotationRightBorder = 0;
	m_iRotationBottomBorder = 0;
}

PaintText::~PaintText()
{
	if (Font)
	{
		DeleteObject(Font);
		Font = NULL;
	}

	if (tmpFont)
	{
		DeleteObject(tmpFont);
		tmpFont = NULL;
	}
}

BOOL PaintText::configure(LPCSTR pszName, LPCSTR pszPrefix, BOOL bUseAlphaMap, const PaintText *pDefaultPaintText, const PaintTextDefaults *pPaintTextDefaults)
{
	string name(pszName);
	string prefix(pszPrefix);

	string strAddToGroup = iGetxRCString(name, (prefix + "FontAddToGroup"), "");

	if (!strAddToGroup.empty() && _stricmp(name.c_str(), strAddToGroup.c_str()) != NULL)
	{	
		return configure(strAddToGroup.c_str(), "", bUseAlphaMap, pDefaultPaintText, pPaintTextDefaults);
	}

	bool bDifferenceToDefault = false;

	const NameValuePair alignValues[] =
	{
		{ ".left", DT_LEFT },
		{ ".center", DT_CENTER },
		{ ".right", DT_RIGHT },
		{ "left", DT_LEFT },
		{ "center", DT_CENTER },
		{ "right", DT_RIGHT },
		{ 0, 0 }
	};

	const NameValuePair vertAlignValues[] =
	{
		{ ".top", DT_TOP },
		{ ".center", DT_VCENTER },
		{ ".bottom", DT_BOTTOM },
		{ "top", DT_TOP },
		{ "center", DT_VCENTER },
		{ "bottom", DT_BOTTOM },
		{ 0, 0 }
	};

	const NameValuePair CharSetConstants[] =
	{
		{ "ANSI_CHARSET", ANSI_CHARSET },
		{ "BALTIC_CHARSET", BALTIC_CHARSET },
		{ "CHINESEBIG5_CHARSET", CHINESEBIG5_CHARSET },
		{ "DEFAULT_CHARSET", DEFAULT_CHARSET },
		{ "EASTEUROPE_CHARSET", EASTEUROPE_CHARSET },
		{ "GB2312_CHARSET", GB2312_CHARSET },
		{ "GREEK_CHARSET", GREEK_CHARSET },
		{ "HANGUL_CHARSET", HANGUL_CHARSET },
		{ "MAC_CHARSET", MAC_CHARSET },
		{ "RUSSIAN_CHARSET", RUSSIAN_CHARSET },
		{ "SHIFTJIS_CHARSET", SHIFTJIS_CHARSET },
		{ "SYMBOL_CHARSET", SYMBOL_CHARSET },
		{ "TURKISH_CHARSET", TURKISH_CHARSET },
		{ "VIETNAMESE_CHARSET", VIETNAMESE_CHARSET },
		{ "JOHAB_CHARSET", JOHAB_CHARSET },
		{ "ARABIC_CHARSET", ARABIC_CHARSET },
		{ "HEBREW_CHARSET", HEBREW_CHARSET },
		{ "THAI_CHARSET", THAI_CHARSET },
		{ "OEM_CHARSET", OEM_CHARSET },
		{ 0, 0 }
	};

	//Create Dummy Default
	bool bDefaultPaintTextCreated = false;
	if ( pDefaultPaintText == NULL )
	{
		bDefaultPaintTextCreated = true;
		if ( pPaintTextDefaults == NULL )
			pDefaultPaintText = new PaintText();
		else
			pDefaultPaintText = new PaintText(pPaintTextDefaults);
	}

	m_bUseAlphaMap = (bUseAlphaMap ? true : false);

	m_bFontVisible = iGetx2RCBool(name, (prefix + "FontVisible"), pDefaultPaintText->m_bFontVisible);
	if ( m_bFontVisible != pDefaultPaintText->m_bFontVisible )
		bDifferenceToDefault = true;

	m_iFontAlphaTransparency = iGetx2RCInt(name, (prefix + "FontAlphaTransparency"), pDefaultPaintText->m_iFontAlphaTransparency, 0, 255);
	if ( m_iFontAlphaTransparency != pDefaultPaintText->m_iFontAlphaTransparency)
		bDifferenceToDefault = true;

	m_strBorders = iGetx2RCLine(name, (prefix + "FontBorders"), pDefaultPaintText->m_strBorders);
	if ( _stricmp(m_strBorders.c_str(), pDefaultPaintText->m_strBorders.c_str()) != NULL )
		bDifferenceToDefault = true;

	if (m_strBorders.empty())
	{
		m_iLeftBorder = iGetx2RCInt(name, (prefix + "FontLeftBorder"), pDefaultPaintText->m_iLeftBorder, 0);
		m_iTopBorder = iGetx2RCInt(name, (prefix + "FontTopBorder"), pDefaultPaintText->m_iTopBorder, 0);
		m_iRightBorder = iGetx2RCInt(name, (prefix + "FontRightBorder"), pDefaultPaintText->m_iRightBorder, 0);
		m_iBottomBorder = iGetx2RCInt(name, (prefix + "FontBottomBorder"), pDefaultPaintText->m_iBottomBorder, 0);
	}
	else
	{
		Parse4SizesCFG(m_strBorders, m_iLeftBorder, m_iRightBorder, m_iTopBorder, m_iBottomBorder, true/*OnlyPositive*/);
		
		m_iLeftBorder = iGetxRCInt(name, (prefix + "FontLeftBorder"), m_iLeftBorder, 0);
		m_iTopBorder = iGetxRCInt(name, (prefix + "FontTopBorder"), m_iTopBorder, 0);
		m_iRightBorder = iGetxRCInt(name, (prefix + "FontRightBorder"), m_iRightBorder, 0);
		m_iBottomBorder = iGetxRCInt(name, (prefix + "FontBottomBorder"), m_iBottomBorder, 0);
	}

		if ( m_iLeftBorder != pDefaultPaintText->m_iLeftBorder)
			bDifferenceToDefault = true;
		if ( m_iRightBorder != pDefaultPaintText->m_iRightBorder)
			bDifferenceToDefault = true;
		if ( m_iTopBorder != pDefaultPaintText->m_iTopBorder)
			bDifferenceToDefault = true;
		if ( m_iBottomBorder != pDefaultPaintText->m_iBottomBorder)
			bDifferenceToDefault = true;

	m_bFontShadow = iGetx2RCBool(name, (prefix + "FontShadow"), pDefaultPaintText->m_bFontShadow);
	if ( m_bFontShadow != pDefaultPaintText->m_bFontShadow )
		bDifferenceToDefault = true;
	m_iFontShadowX = iGetx2RCInt(name, (prefix + "FontShadowX"), pDefaultPaintText->m_iFontShadowX);
	if ( m_iFontShadowX != pDefaultPaintText->m_iFontShadowX )
		bDifferenceToDefault = true;
	m_iFontShadowY = iGetx2RCInt(name, (prefix + "FontShadowY"), pDefaultPaintText->m_iFontShadowY);
	if ( m_iFontShadowY != pDefaultPaintText->m_iFontShadowY )
		bDifferenceToDefault = true;
	m_iFontBlockedShadow = iGetx2RCInt(name, (prefix + "FontShadowBlocked"), pDefaultPaintText->m_iFontBlockedShadow, -10, 10);
	if ( m_iFontBlockedShadow != pDefaultPaintText->m_iFontBlockedShadow )
		bDifferenceToDefault = true;
	m_clrFontShadowColor = iGetx2RCColor(name, (prefix + "FontShadowColor"), pDefaultPaintText->m_clrFontShadowColor);
	if ( m_clrFontShadowColor != pDefaultPaintText->m_clrFontShadowColor )
		bDifferenceToDefault = true;

	m_bFontOutline = iGetx2RCBool(name, (prefix + "FontOutline"), pDefaultPaintText->m_bFontOutline);
	if ( m_bFontOutline != pDefaultPaintText->m_bFontOutline )
		bDifferenceToDefault = true;
	m_clrFontOutlineColor = iGetx2RCColor(name, (prefix + "FontOutlineColor"), pDefaultPaintText->m_clrFontOutlineColor);
	if ( m_clrFontOutlineColor != pDefaultPaintText->m_clrFontOutlineColor )
		bDifferenceToDefault = true;
	m_bFontOutlineExtra = iGetx2RCBool(name, (prefix + "FontOutlineExtra"), pDefaultPaintText->m_bFontOutlineExtra);
	if ( m_bFontOutlineExtra != pDefaultPaintText->m_bFontOutlineExtra )
		bDifferenceToDefault = true;

	m_clrFontEmbossColor1 = iGetx2RCColor(name, (prefix + "FontEmbossColor1"), pDefaultPaintText->m_clrFontEmbossColor1);
	if ( m_clrFontEmbossColor1 != pDefaultPaintText->m_clrFontEmbossColor1)
		bDifferenceToDefault = true;
	m_clrFontEmbossColor2 = iGetx2RCColor(name, (prefix + "FontEmbossColor2"), m_clrFontEmbossColor1);
	if ( m_clrFontEmbossColor2 != pDefaultPaintText->m_clrFontEmbossColor2)
		bDifferenceToDefault = true;

	m_strFontName = iGetx2RCString(name, (prefix + "Font"), pDefaultPaintText->m_strFontName);
	if ( _stricmp(m_strFontName.c_str(), pDefaultPaintText->m_strFontName.c_str()) != NULL )
		bDifferenceToDefault = true;
	m_iFontHeight = iGetx2RCInt(name, (prefix + "FontHeight"), pDefaultPaintText->m_iFontHeight, 0);
	if ( m_iFontHeight != pDefaultPaintText->m_iFontHeight)
		bDifferenceToDefault = true;
	m_clrFontColor = iGetx2RCColor(name, (prefix + "FontColor"), pDefaultPaintText->m_clrFontColor);
	if ( m_clrFontColor != pDefaultPaintText->m_clrFontColor)
		bDifferenceToDefault = true;

	m_shFontCharSet = iGetx2RCNamedValue(name, (prefix + "FontCharSet"), CharSetConstants, pDefaultPaintText->m_shFontCharSet );
	if ( m_shFontCharSet != pDefaultPaintText->m_shFontCharSet)
		bDifferenceToDefault = true;

	m_bFontSmoothing = iGetx2RCBool(name, (prefix + "FontSmoothing"), pDefaultPaintText->m_bFontSmoothing);
	if ( m_bFontSmoothing != pDefaultPaintText->m_bFontSmoothing)
		bDifferenceToDefault = true;
	m_bFontClearType = iGetx2RCBool(name, (prefix + "FontClearType"), pDefaultPaintText->m_bFontClearType);
	if ( m_bFontClearType != pDefaultPaintText->m_bFontClearType)
		bDifferenceToDefault = true;

	m_strFontFadeCFG = iGetx2RCLine(name, (prefix + "FontTextFade"), pDefaultPaintText->m_strFontFadeCFG);
	if ( _stricmp(m_strFontFadeCFG.c_str(), pDefaultPaintText->m_strFontFadeCFG.c_str()) != NULL )
		bDifferenceToDefault = true;

	if (m_strFontFadeCFG.empty())
	{
		m_iLeftFade = iGetx2RCInt(name, (prefix + "FontLeftTextFade"), pDefaultPaintText->m_iLeftFade, 0);
		m_iTopFade = iGetx2RCInt(name, (prefix + "FontTopTextFade"), pDefaultPaintText->m_iTopFade, 0);
		m_iRightFade = iGetx2RCInt(name, (prefix + "FontRightTextFade"), pDefaultPaintText->m_iRightFade, 0);
		m_iBottomFade = iGetx2RCInt(name, (prefix + "FontBottomTextFade"), pDefaultPaintText->m_iBottomFade, 0);
	}
	else
	{
		m_bFontFade = Parse4SizesCFG(m_strFontFadeCFG, m_iLeftFade, m_iRightFade, m_iTopFade, m_iBottomFade, true/*OnlyPositive*/);
		
		m_iLeftFade = iGetxRCInt(name, (prefix + "FontLeftTextFade"), m_iLeftFade, 0);
		m_iTopFade = iGetxRCInt(name, (prefix + "FontTopTextFade"), m_iTopFade, 0);
		m_iRightFade = iGetxRCInt(name, (prefix + "FontRightTextFade"), m_iRightFade, 0);
		m_iBottomFade = iGetxRCInt(name, (prefix + "FontBottomTextFade"), m_iBottomFade, 0);
	}

		if ( m_iLeftFade != pDefaultPaintText->m_iLeftFade )
			bDifferenceToDefault = true;
		if ( m_iTopFade != pDefaultPaintText->m_iTopFade )
			bDifferenceToDefault = true;
		if ( m_iRightFade != pDefaultPaintText->m_iRightFade )
			bDifferenceToDefault = true;
		if ( m_iBottomFade != pDefaultPaintText->m_iBottomFade )
			bDifferenceToDefault = true;
	
	m_bFontBold = iGetx2RCBool(name, (prefix + "FontBold"), pDefaultPaintText->m_bFontBold);
	if ( m_bFontBold != pDefaultPaintText->m_bFontBold)
		bDifferenceToDefault = true;
	m_bFontItalic = iGetx2RCBool(name, (prefix + "FontItalic"), pDefaultPaintText->m_bFontItalic);
	if ( m_bFontItalic != pDefaultPaintText->m_bFontItalic)
		bDifferenceToDefault = true;
	m_bFontUnderline = iGetx2RCBool(name, (prefix + "FontUnderline"), pDefaultPaintText->m_bFontUnderline);
	if ( m_bFontUnderline != pDefaultPaintText->m_bFontUnderline)
		bDifferenceToDefault = true;

	m_iFontAlign = iGetx2RCNamedValue(name, (prefix + "FontAlign"), alignValues, pDefaultPaintText->m_iFontAlign);
	if ( m_iFontAlign != pDefaultPaintText->m_iFontAlign)
		bDifferenceToDefault = true;
	m_iFontVertAlign = iGetx2RCNamedValue(name, (prefix + "FontVertAlign"), vertAlignValues, pDefaultPaintText->m_iFontVertAlign);
	if ( m_iFontVertAlign != pDefaultPaintText->m_iFontVertAlign)
		bDifferenceToDefault = true;
	m_bFontNoEllipsis = iGetx2RCBool(name, (prefix + "FontNoEllipsis"), pDefaultPaintText->m_bFontNoEllipsis);
	if ( m_bFontNoEllipsis != pDefaultPaintText->m_bFontNoEllipsis)
		bDifferenceToDefault = true;

	//Update 4
	m_iFontRotation = iGetx2RCInt(name, (prefix + "FontRotation"), pDefaultPaintText->m_iFontRotation, -360, 360);
	if ( m_iFontRotation <= -360 || m_iFontRotation >= 360 )
		m_iFontRotation = 0;

	if ( m_iFontRotation != pDefaultPaintText->m_iFontRotation)
		bDifferenceToDefault = true;

	if ( m_iFontRotation != 0 )
	{
		m_iRotationLeftBorder = m_iLeftBorder;
		m_iRotationTopBorder = m_iTopBorder;
		m_iRotationRightBorder = m_iRightBorder;
		m_iRotationBottomBorder = m_iBottomBorder;

		m_iLeftBorder = 0;
		m_iTopBorder = 0;
		m_iRightBorder = 0;
		m_iBottomBorder = 0;
	}

	if ( bDefaultPaintTextCreated == true )
		delete pDefaultPaintText;

	CreateFontHandle(Font, m_strFontName, m_iFontHeight, m_bFontItalic, m_bFontUnderline, m_bFontBold, m_shFontCharSet, m_bFontSmoothing, m_bFontClearType);

	return bDifferenceToDefault;
}

void PaintText::apply(HDC desthDC, int x, int y, int width, int height, LPCSTR pszDrawText, BOOL bMultiLine, BOOL bSetScrolling, BOOL bDisableExtras)
{
	if ( width <= 0 || height <= 0 || !m_bFontVisible )
		return;

	RECT drawRect;
	drawRect.left = x + m_iLeftBorder;
	drawRect.top = y + m_iTopBorder;
	drawRect.right = x + width - m_iRightBorder;
	drawRect.bottom = y + height - m_iBottomBorder;

	if ( drawRect.right <= drawRect.left || drawRect.bottom <= drawRect.top )
		return;

	string drawText = pszDrawText;

	if (bDisableExtras || (!m_bUseAlphaMap && !m_bFontFade && m_iFontAlphaTransparency == 255 && m_iFontRotation == 0))
	{
		iapply(desthDC, x, y, width, height, drawText, (bMultiLine ? true : false), (bSetScrolling ? true : false));
		return;
	}

	int iBaseWidth = width;
	int iBaseHeight = height;

	if ( m_iFontRotation != 0 )
	{ 
		if ( (m_iFontRotation >= 45 && m_iFontRotation <= 135) || (m_iFontRotation >= 225 && m_iFontRotation <= 315) || (m_iFontRotation <= -45 && m_iFontRotation >= -135) || (m_iFontRotation <= -225 && m_iFontRotation >= -315) )
		{
			int iTmpWidht = width;
			width = height;
			height = width;
		}
	}

	BITMAPINFO alpha;
	ZeroMemory( &alpha.bmiHeader, sizeof(BITMAPINFOHEADER) );
	alpha.bmiHeader.biWidth=width;      // Set size you need
	alpha.bmiHeader.biHeight=height;    // Set size you need
	alpha.bmiHeader.biPlanes=1;
	alpha.bmiHeader.biBitCount=32;      // Can be 8, 16, 32 bpp or other number
	alpha.bmiHeader.biSizeImage=0;
	alpha.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
	alpha.bmiHeader.biClrUsed= 0;
	alpha.bmiHeader.biClrImportant= 0;
	VOID *pvBits;

	HDC memoryDC = CreateCompatibleDC(NULL);

	HBITMAP memoryBitmap = CreateDIBSection( memoryDC,
												&alpha,
												DIB_RGB_COLORS,
												&pvBits,
												NULL,
												0 );

	HBITMAP oldmemoryBitmap = (HBITMAP) SelectObject(memoryDC, memoryBitmap);

	BITMAP bm_memory;
	GetObject(memoryBitmap, sizeof(BITMAP), &bm_memory);

	BYTE *pPixel_memory = (LPBYTE)bm_memory.bmBits;

	int iSize = width * height;

	for (int j = 0; j < iSize; ++j)
	{
		pPixel_memory[0] = 0;
		pPixel_memory[1] = 0;
		pPixel_memory[2] = 0;
		pPixel_memory[3] = 0;

		pPixel_memory+= 4;
	}

	HDC darkDC = CreateCompatibleDC(NULL);

	HBITMAP darkBitmap = CreateDIBSection( darkDC,
												&alpha,
												DIB_RGB_COLORS,
												&pvBits,
												NULL,
												0 );

	HBITMAP olddarkBitmap = (HBITMAP) SelectObject(darkDC, darkBitmap);

	HDC lightDC = CreateCompatibleDC(NULL);

	HBITMAP lightBitmap = CreateDIBSection( lightDC,
												&alpha,
												DIB_RGB_COLORS,
												&pvBits,
												NULL,
												0 );

	HBITMAP oldlightBitmap = (HBITMAP) SelectObject(lightDC, lightBitmap);

	BITMAP bm_dark;
	GetObject(darkBitmap, sizeof(BITMAP), &bm_dark);

	BYTE *pPixel_dark = (LPBYTE)bm_dark.bmBits;

	for (j = 0; j < iSize; ++j)
	{
		pPixel_dark[0] = 0;
		pPixel_dark[1] = 0;
		pPixel_dark[2] = 0;
		pPixel_dark[3] = 255;

		pPixel_dark+= 4;
	}

	BITMAP bm_light;
	GetObject(lightBitmap, sizeof(BITMAP), &bm_light);

	BYTE *pPixel_light = (LPBYTE)bm_light.bmBits;

	for (j = 0; j < iSize; ++j)
	{
		pPixel_light[0] = 255;
		pPixel_light[1] = 255;
		pPixel_light[2] = 255;
		pPixel_light[3] = 255;

		pPixel_light+= 4;
	}

	int iRepeatNeeded = 0;

	if ( (m_bFontShadow || m_iFontBlockedShadow != 0) && !m_bFontOutline )
	{
		iapplyColorShadow(darkDC, 0, 0, width, height, drawText.c_str(), (bMultiLine ? true : false), (bSetScrolling ? true : false));
		iapplyColorShadow(lightDC, 0, 0, width, height, drawText.c_str(), (bMultiLine ? true : false), (bSetScrolling ? true : false));
	
		iRepeatNeeded = 1;
	}
	else if ( m_bFontOutline )
	{
		iapplyColorOutline(darkDC, 0, 0, width, height, drawText.c_str(), (bMultiLine ? true : false), (bSetScrolling ? true : false));
		iapplyColorOutline(lightDC, 0, 0, width, height, drawText.c_str(), (bMultiLine ? true : false), (bSetScrolling ? true : false));
	
		iRepeatNeeded = 2;
	}
	else if ( m_clrFontEmbossColor1 != RGB(255,0,255) )
	{
		iapply(darkDC, 0, 0, width, height, drawText.c_str(), (bMultiLine ? true : false), (bSetScrolling ? true : false));
		iapply(lightDC, 0, 0, width, height, drawText.c_str(), (bMultiLine ? true : false), (bSetScrolling ? true : false));

		iRepeatNeeded = -1;
	}
	else
	{
		iapplyColor(darkDC, 0, 0, width, height, drawText.c_str(), (bMultiLine ? true : false), (bSetScrolling ? true : false));
		iapplyColor(lightDC, 0, 0, width, height, drawText.c_str(), (bMultiLine ? true : false), (bSetScrolling ? true : false));
	
		iRepeatNeeded = 0;
	}

	GetObject(darkBitmap, sizeof(BITMAP), &bm_dark);
	pPixel_dark = (LPBYTE)bm_dark.bmBits;

	GetObject(lightBitmap, sizeof(BITMAP), &bm_light);
	pPixel_light = (LPBYTE)bm_light.bmBits;

	int tmpx = 0;
	int tmpy = 0;
	int xval = 255;
	int yval = 255;

	int iLeftFade = (m_iLeftFade != 0 ? 255/m_iLeftFade : 0);
	int iRightFade = (m_iRightFade != 0 ? 255/m_iRightFade : 0);
	int iTopFade = (m_iTopFade != 0 ? 255/m_iTopFade : 0);
	int iBottomFade = (m_iBottomFade != 0 ? 255/m_iBottomFade : 0);

	for (j = 0; j < iSize; ++j)
	{
		if (pPixel_dark[3] == 0)
		{
			xval = 255;
			yval = 255;

			if ( m_bFontFade )
			{
				tmpx = (j+1)%width;
				tmpy = j/width;

				if ( tmpx == 0 )
					tmpx = width-1;

				if ( m_iLeftFade != 0 && tmpx > m_iLeftBorder && tmpx <= m_iLeftBorder+m_iLeftFade )
					xval = iLeftFade*(tmpx-m_iLeftBorder);
				else if ( m_iRightFade != 0 && tmpx > width-m_iRightBorder-m_iRightFade && tmpx <= width-m_iRightBorder )
					xval = iRightFade*(width-m_iRightBorder-tmpx);
				
				//Order is reverted Top<->Bottom
				if ( m_iBottomFade != 0 && tmpy > m_iBottomBorder && tmpy < m_iBottomBorder+m_iBottomFade )
					yval = iBottomFade*(tmpy-m_iBottomBorder);
				else if ( m_iTopFade != 0 && tmpy >= height-m_iTopBorder-m_iTopFade && tmpy < height-m_iTopBorder )
					yval = iTopFade*(height-m_iTopBorder-tmpy);
				
				if ( xval == 255 )
					pPixel_dark[3] = yval;
				else if ( yval == 255 )
					pPixel_dark[3] = xval;
				else
					pPixel_dark[3] = (xval*yval)/255;

				pPixel_dark[3] = pPixel_dark[3] - ((pPixel_light[0] - pPixel_dark[0]) * pPixel_dark[3] / 255);
			}
			else
			{
				if (pPixel_dark[0] == pPixel_light[0] && pPixel_dark[1] == pPixel_light[1] && pPixel_dark[2] == pPixel_light[2])
					pPixel_dark[3] = 255;
				else if (pPixel_dark[0] == 0 && pPixel_dark[1] == 0 && pPixel_dark[2] == 0 && pPixel_light[0] == 255 && pPixel_light[1] == 255 && pPixel_light[2] == 255 )
					pPixel_dark[3] = 0;
				else
					pPixel_dark[3] = 255 - (pPixel_light[0] - pPixel_dark[0]);
			}

			if ( pPixel_dark[0] == 255 && pPixel_dark[1] == 0 && pPixel_dark[2] == 255 )
			{
				pPixel_dark[0] = 0;
				pPixel_dark[1] = 0;
				pPixel_dark[2] = 0;
				pPixel_dark[3] = 0;
			}
			else
			{
				if ( iRepeatNeeded == 1 )
				{	
					//PreMultiply
					pPixel_dark[0] = GetBValue(m_clrFontShadowColor) * pPixel_dark[3] / 255;
					pPixel_dark[1] = GetGValue(m_clrFontShadowColor) * pPixel_dark[3] / 255;
					pPixel_dark[2] = GetRValue(m_clrFontShadowColor) * pPixel_dark[3] / 255;
				}
				else if ( iRepeatNeeded == 2 )
				{
					//PreMultiply
					pPixel_dark[0] = GetBValue(m_clrFontOutlineColor) * pPixel_dark[3] / 255;
					pPixel_dark[1] = GetGValue(m_clrFontOutlineColor) * pPixel_dark[3] / 255;
					pPixel_dark[2] = GetRValue(m_clrFontOutlineColor) * pPixel_dark[3] / 255;
				}
				else if ( iRepeatNeeded == -1 )
				{
					//PreMultiply
					pPixel_dark[0] = pPixel_dark[0] * pPixel_dark[3] / 255;
					pPixel_dark[1] = pPixel_dark[1] * pPixel_dark[3] / 255;
					pPixel_dark[2] = pPixel_dark[2] * pPixel_dark[3] / 255;
				}
				else
				{
					//PreMultiply
					pPixel_dark[0] = GetBValue(m_clrFontColor) * pPixel_dark[3] / 255;
					pPixel_dark[1] = GetGValue(m_clrFontColor) * pPixel_dark[3] / 255;
					pPixel_dark[2] = GetRValue(m_clrFontColor) * pPixel_dark[3] / 255;
				}
			}
		}
		else
		{
			pPixel_dark[0] = 0;
			pPixel_dark[1] = 0;
			pPixel_dark[2] = 0;
			pPixel_dark[3] = 0;
		}
					
		pPixel_dark+= 4;	
		pPixel_light+= 4;	
	}

	if ( iRepeatNeeded > 0)
	{
		BLENDFUNCTION blendFunction = { AC_SRC_OVER, 0, 255, AC_SRC_ALPHA };

		AlphaBlend(memoryDC, 0, 0, width, height, darkDC, 0, 0, width, height, blendFunction);

		GetObject(darkBitmap, sizeof(BITMAP), &bm_dark);

		pPixel_dark = (LPBYTE)bm_dark.bmBits;

		iSize = width * height;

		for (int j = 0; j < iSize; ++j)
		{
			pPixel_dark[0] = 0;
			pPixel_dark[1] = 0;
			pPixel_dark[2] = 0;
			pPixel_dark[3] = 255;

			pPixel_dark+= 4;
		}

		GetObject(lightBitmap, sizeof(BITMAP), &bm_light);

		pPixel_light = (LPBYTE)bm_light.bmBits;

		for (j = 0; j < iSize; ++j)
		{
			pPixel_light[0] = 255;
			pPixel_light[1] = 255;
			pPixel_light[2] = 255;
			pPixel_light[3] = 255;

			pPixel_light+= 4;
		}

		iapplyColor(darkDC, 0, 0, width, height, drawText.c_str(), (bMultiLine ? true : false), (bSetScrolling ? true : false));
		iapplyColor(lightDC, 0, 0, width, height, drawText.c_str(), (bMultiLine ? true : false), (bSetScrolling ? true : false));

		GetObject(darkBitmap, sizeof(BITMAP), &bm_dark);
		pPixel_dark = (LPBYTE)bm_dark.bmBits;

		GetObject(lightBitmap, sizeof(BITMAP), &bm_light);
		pPixel_light = (LPBYTE)bm_light.bmBits;

		int tmpx = 0;
		int tmpy = 0;
		int xval = 255;
		int yval = 255;

		int iLeftFade = (m_iLeftFade != 0 ? 255/m_iLeftFade : 0);
		int iRightFade = (m_iRightFade != 0 ? 255/m_iRightFade : 0);
		int iTopFade = (m_iTopFade != 0 ? 255/m_iTopFade : 0);
		int iBottomFade = (m_iBottomFade != 0 ? 255/m_iBottomFade : 0);

		for (j = 0; j < iSize; ++j)
		{
			if (pPixel_dark[3] == 0)
			{
				xval = 255;
				yval = 255;

				if ( m_bFontFade )
				{
					tmpx = (j+1)%width;
					tmpy = j/width;

					if ( tmpx == 0 )
						tmpx = width-1;

					if ( m_iLeftFade != 0 && tmpx > m_iLeftBorder && tmpx <= m_iLeftBorder+m_iLeftFade )
						xval = iLeftFade*(tmpx-m_iLeftBorder);
					else if ( m_iRightFade != 0 && tmpx > width-m_iRightBorder-m_iRightFade && tmpx <= width-m_iRightBorder )
						xval = iRightFade*(width-m_iRightBorder-tmpx);
					
					//Order is reverted Top<->Bottom
					if ( m_iBottomFade != 0 && tmpy > m_iBottomBorder && tmpy < m_iBottomBorder+m_iBottomFade )
						yval = iBottomFade*(tmpy-m_iBottomBorder);
					else if ( m_iTopFade != 0 && tmpy >= height-m_iTopBorder-m_iTopFade && tmpy < height-m_iTopBorder )
						yval = iTopFade*(height-m_iTopBorder-tmpy);
					
					if ( xval == 255 )
						pPixel_dark[3] = yval;
					else if ( yval == 255 )
						pPixel_dark[3] = xval;
					else
						pPixel_dark[3] = (xval*yval)/255;

					pPixel_dark[3] = pPixel_dark[3] - ((pPixel_light[0] - pPixel_dark[0]) * pPixel_dark[3] / 255);
				}
				else
				{
					if (pPixel_dark[0] == pPixel_light[0] && pPixel_dark[1] == pPixel_light[1] && pPixel_dark[2] == pPixel_light[2])
						pPixel_dark[3] = 255;
					else if (pPixel_dark[0] == 0 && pPixel_dark[1] == 0 && pPixel_dark[2] == 0 && pPixel_light[0] == 255 && pPixel_light[1] == 255 && pPixel_light[2] == 255 )
						pPixel_dark[3] = 0;
					else
						pPixel_dark[3] = 255 - (pPixel_light[0] - pPixel_dark[0]);
				}

				if ( pPixel_dark[0] == 255 && pPixel_dark[1] == 0 && pPixel_dark[2] == 255 )
				{
					pPixel_dark[0] = 0;
					pPixel_dark[1] = 0;
					pPixel_dark[2] = 0;
					pPixel_dark[3] = 0;
				}
				else
				{
					//PreMultiply
					pPixel_dark[0] = GetBValue(m_clrFontColor) * pPixel_dark[3] / 255;
					pPixel_dark[1] = GetGValue(m_clrFontColor) * pPixel_dark[3] / 255;
					pPixel_dark[2] = GetRValue(m_clrFontColor) * pPixel_dark[3] / 255;
				}
			}
			else
			{
				pPixel_dark[0] = 0;
				pPixel_dark[1] = 0;
				pPixel_dark[2] = 0;
				pPixel_dark[3] = 0;
			}
						
			pPixel_dark+= 4;	
			pPixel_light+= 4;	
		}

		AlphaBlend(memoryDC, 0, 0, width, height, darkDC, 0, 0, width, height, blendFunction);
	}
	else
	{
		BLENDFUNCTION blendFunction = { AC_SRC_OVER, 0, 255, AC_SRC_ALPHA };

		AlphaBlend(memoryDC, 0, 0, width, height, darkDC, 0, 0, width, height, blendFunction);
	}

	if ( m_iFontRotation != 0 )
	{
		HBITMAP newmemoryBitmap = aarot::rotate(memoryBitmap, m_iFontRotation);

		int iWidth = width;
		int iHeight = height;
		GetLSBitmapSize(newmemoryBitmap, &iWidth, &iHeight);

		int iDX = (iBaseWidth - iWidth) / 2;
		int iDY = (iBaseHeight - iHeight) / 2;

		memoryBitmap = (HBITMAP) SelectObject(memoryDC, newmemoryBitmap);

		BLENDFUNCTION blendFunction = { AC_SRC_OVER, 0, m_iFontAlphaTransparency, AC_SRC_ALPHA };

		AlphaBlend(desthDC, x+iDX, y+iDY, iWidth, iHeight, memoryDC, 0, 0, iWidth, iHeight, blendFunction);
	}
	else
	{
		BLENDFUNCTION blendFunction = { AC_SRC_OVER, 0, m_iFontAlphaTransparency, AC_SRC_ALPHA };

		AlphaBlend(desthDC, x, y, width, height, memoryDC, 0, 0, width, height, blendFunction);
	}

	memoryBitmap = (HBITMAP) SelectObject(memoryDC, oldmemoryBitmap);
	DeleteDC(memoryDC);
	DeleteObject(memoryBitmap);
	DeleteObject(oldmemoryBitmap);

	darkBitmap = (HBITMAP) SelectObject(darkDC, olddarkBitmap);
	DeleteDC(darkDC);
	DeleteObject(darkBitmap);
	DeleteObject(olddarkBitmap);

	lightBitmap = (HBITMAP) SelectObject(lightDC, oldlightBitmap);
	DeleteDC(lightDC);
	DeleteObject(lightBitmap);
	DeleteObject(oldlightBitmap);
}

void PaintText::iapply(HDC desthDC, int x, int y, int width, int height, const string &drawText, bool bMultiLine, bool bSetScrolling)
{
	RECT drawRect;
	drawRect.left = x + m_iLeftBorder+m_iRotationLeftBorder;
	drawRect.top = y + m_iTopBorder+m_iRotationTopBorder;
	drawRect.right = x + width - m_iRightBorder-m_iRotationRightBorder;
	drawRect.bottom = y + height - m_iBottomBorder-m_iRotationBottomBorder;

	int bkMode = SetBkMode(desthDC, TRANSPARENT);
	int textColor = SetTextColor(desthDC, m_clrFontColor);

	tmpFont = (HFONT) SelectObject(desthDC, Font);

	if (m_clrFontEmbossColor1 != RGB(255,0,255))
	{
		HFONT disabledFont = NULL;
		CreateFontHandle(disabledFont, m_strFontName, m_iFontHeight, m_bFontItalic, m_bFontUnderline, m_bFontBold, m_shFontCharSet, false, false);

		Font = (HFONT) SelectObject(desthDC, disabledFont);

		SetTextColor(desthDC, m_clrFontEmbossColor1);
		OffsetRect(&drawRect, 0, -1);
		DrawTextEx(desthDC, (char *) drawText.c_str(), drawText.length(), &drawRect, (bSetScrolling ? DT_LEFT : m_iFontAlign) | (bMultiLine ? DT_WORDBREAK : DT_SINGLELINE) | m_iFontVertAlign | ((m_bFontNoEllipsis || bSetScrolling) ? 0 : DT_END_ELLIPSIS), 0);
		OffsetRect(&drawRect, -1, 0);
		DrawTextEx(desthDC, (char *) drawText.c_str(), drawText.length(), &drawRect, (bSetScrolling ? DT_LEFT : m_iFontAlign) | (bMultiLine ? DT_WORDBREAK : DT_SINGLELINE) | m_iFontVertAlign | ((m_bFontNoEllipsis || bSetScrolling) ? 0 : DT_END_ELLIPSIS), 0);
		OffsetRect(&drawRect, 0, 1);
		DrawTextEx(desthDC, (char *) drawText.c_str(), drawText.length(), &drawRect, (bSetScrolling ? DT_LEFT : m_iFontAlign) | (bMultiLine ? DT_WORDBREAK : DT_SINGLELINE) | m_iFontVertAlign | ((m_bFontNoEllipsis || bSetScrolling) ? 0 : DT_END_ELLIPSIS), 0);
		
		SetTextColor(desthDC, m_clrFontEmbossColor2);
		OffsetRect(&drawRect, 1, 1);
		DrawTextEx(desthDC, (char *) drawText.c_str(), drawText.length(), &drawRect, (bSetScrolling ? DT_LEFT : m_iFontAlign) | (bMultiLine ? DT_WORDBREAK : DT_SINGLELINE) | m_iFontVertAlign | ((m_bFontNoEllipsis || bSetScrolling) ? 0 : DT_END_ELLIPSIS), 0);
		OffsetRect(&drawRect, 1, 0);
		DrawTextEx(desthDC, (char *) drawText.c_str(), drawText.length(), &drawRect, (bSetScrolling ? DT_LEFT : m_iFontAlign) | (bMultiLine ? DT_WORDBREAK : DT_SINGLELINE) | m_iFontVertAlign | ((m_bFontNoEllipsis || bSetScrolling) ? 0 : DT_END_ELLIPSIS), 0);
		OffsetRect(&drawRect, 0, -1);
		DrawTextEx(desthDC, (char *) drawText.c_str(), drawText.length(), &drawRect, (bSetScrolling ? DT_LEFT : m_iFontAlign) | (bMultiLine ? DT_WORDBREAK : DT_SINGLELINE) | m_iFontVertAlign | ((m_bFontNoEllipsis || bSetScrolling) ? 0 : DT_END_ELLIPSIS), 0);

		SetTextColor(desthDC, RGB(255,0,255));
		OffsetRect(&drawRect, -1, 0);
		DrawTextEx(desthDC, (char *) drawText.c_str(), drawText.length(), &drawRect, (bSetScrolling ? DT_LEFT : m_iFontAlign) | (bMultiLine ? DT_WORDBREAK : DT_SINGLELINE) | m_iFontVertAlign | ((m_bFontNoEllipsis || bSetScrolling) ? 0 : DT_END_ELLIPSIS), 0);
	
		disabledFont = (HFONT) SelectObject(desthDC, tmpFont);
		if ( disabledFont )
			DeleteObject(disabledFont);
	}
	else
	{	
		if ( m_iFontBlockedShadow != 0 && !m_bFontOutline )
		{
			SetTextColor(desthDC, m_clrFontShadowColor);

			for (int i = m_iFontBlockedShadow-1; i >= 0; i--)
			{
				OffsetRect(&drawRect, -1, 1);
				DrawTextEx(desthDC, (char *) drawText.c_str(), drawText.length(), &drawRect, (bSetScrolling ? DT_LEFT : m_iFontAlign) | (bMultiLine ? DT_WORDBREAK : DT_SINGLELINE) | m_iFontVertAlign | ((m_bFontNoEllipsis || bSetScrolling) ? 0 : DT_END_ELLIPSIS), 0);
			}
			
			SetTextColor(desthDC, m_clrFontColor);
		}
		else if (m_bFontShadow && !m_bFontOutline)
		{
			SetTextColor(desthDC, m_clrFontShadowColor);

			OffsetRect(&drawRect, m_iFontShadowX, m_iFontShadowY);
			DrawTextEx(desthDC, (char *) drawText.c_str(), drawText.length(), &drawRect, (bSetScrolling ? DT_LEFT : m_iFontAlign) | (bMultiLine ? DT_WORDBREAK : DT_SINGLELINE) | m_iFontVertAlign | ((m_bFontNoEllipsis || bSetScrolling) ? 0 : DT_END_ELLIPSIS), 0);

			SetTextColor(desthDC, m_clrFontColor);
		}
		else if (m_bFontOutline)
		{
			SetTextColor(desthDC, m_clrFontOutlineColor);

			if ( !m_bFontOutlineExtra )
			{
				OffsetRect(&drawRect, 1, 0);
				DrawTextEx(desthDC, (char *) drawText.c_str(), drawText.length(), &drawRect, (bSetScrolling ? DT_LEFT : m_iFontAlign) | (bMultiLine ? DT_WORDBREAK : DT_SINGLELINE) | m_iFontVertAlign | ((m_bFontNoEllipsis || bSetScrolling) ? 0 : DT_END_ELLIPSIS), 0);

				OffsetRect(&drawRect, -2, 0);
				DrawTextEx(desthDC, (char *) drawText.c_str(), drawText.length(), &drawRect, (bSetScrolling ? DT_LEFT : m_iFontAlign) | (bMultiLine ? DT_WORDBREAK : DT_SINGLELINE) | m_iFontVertAlign | ((m_bFontNoEllipsis || bSetScrolling) ? 0 : DT_END_ELLIPSIS), 0);

				OffsetRect(&drawRect, 1, -1);
				DrawTextEx(desthDC, (char *) drawText.c_str(), drawText.length(), &drawRect, (bSetScrolling ? DT_LEFT : m_iFontAlign) | (bMultiLine ? DT_WORDBREAK : DT_SINGLELINE) | m_iFontVertAlign | ((m_bFontNoEllipsis || bSetScrolling) ? 0 : DT_END_ELLIPSIS), 0);

				OffsetRect(&drawRect, 0, 2);
				DrawTextEx(desthDC, (char *) drawText.c_str(), drawText.length(), &drawRect, (bSetScrolling ? DT_LEFT : m_iFontAlign) | (bMultiLine ? DT_WORDBREAK : DT_SINGLELINE) | m_iFontVertAlign | ((m_bFontNoEllipsis || bSetScrolling) ? 0 : DT_END_ELLIPSIS), 0);
			}
			else
			{
				OffsetRect(&drawRect, 1, 1);
				DrawTextEx(desthDC, (char *) drawText.c_str(), drawText.length(), &drawRect, (bSetScrolling ? DT_LEFT : m_iFontAlign) | (bMultiLine ? DT_WORDBREAK : DT_SINGLELINE) | m_iFontVertAlign | ((m_bFontNoEllipsis || bSetScrolling) ? 0 : DT_END_ELLIPSIS), 0);

				OffsetRect(&drawRect, -1, 0);
				DrawTextEx(desthDC, (char *) drawText.c_str(), drawText.length(), &drawRect, (bSetScrolling ? DT_LEFT : m_iFontAlign) | (bMultiLine ? DT_WORDBREAK : DT_SINGLELINE) | m_iFontVertAlign | ((m_bFontNoEllipsis || bSetScrolling) ? 0 : DT_END_ELLIPSIS), 0);

				OffsetRect(&drawRect, -1, 0);
				DrawTextEx(desthDC, (char *) drawText.c_str(), drawText.length(), &drawRect, (bSetScrolling ? DT_LEFT : m_iFontAlign) | (bMultiLine ? DT_WORDBREAK : DT_SINGLELINE) | m_iFontVertAlign | ((m_bFontNoEllipsis || bSetScrolling) ? 0 : DT_END_ELLIPSIS), 0);

				OffsetRect(&drawRect, 0, -1);
				DrawTextEx(desthDC, (char *) drawText.c_str(), drawText.length(), &drawRect, (bSetScrolling ? DT_LEFT : m_iFontAlign) | (bMultiLine ? DT_WORDBREAK : DT_SINGLELINE) | m_iFontVertAlign | ((m_bFontNoEllipsis || bSetScrolling) ? 0 : DT_END_ELLIPSIS), 0);

				OffsetRect(&drawRect, 0, -1);
				DrawTextEx(desthDC, (char *) drawText.c_str(), drawText.length(), &drawRect, (bSetScrolling ? DT_LEFT : m_iFontAlign) | (bMultiLine ? DT_WORDBREAK : DT_SINGLELINE) | m_iFontVertAlign | ((m_bFontNoEllipsis || bSetScrolling) ? 0 : DT_END_ELLIPSIS), 0);

				OffsetRect(&drawRect, 1, 0);
				DrawTextEx(desthDC, (char *) drawText.c_str(), drawText.length(), &drawRect, (bSetScrolling ? DT_LEFT : m_iFontAlign) | (bMultiLine ? DT_WORDBREAK : DT_SINGLELINE) | m_iFontVertAlign | ((m_bFontNoEllipsis || bSetScrolling) ? 0 : DT_END_ELLIPSIS), 0);

				OffsetRect(&drawRect, 1, 0);
				DrawTextEx(desthDC, (char *) drawText.c_str(), drawText.length(), &drawRect, (bSetScrolling ? DT_LEFT : m_iFontAlign) | (bMultiLine ? DT_WORDBREAK : DT_SINGLELINE) | m_iFontVertAlign | ((m_bFontNoEllipsis || bSetScrolling) ? 0 : DT_END_ELLIPSIS), 0);

				OffsetRect(&drawRect, 0, 1);
				DrawTextEx(desthDC, (char *) drawText.c_str(), drawText.length(), &drawRect, (bSetScrolling ? DT_LEFT : m_iFontAlign) | (bMultiLine ? DT_WORDBREAK : DT_SINGLELINE) | m_iFontVertAlign | ((m_bFontNoEllipsis || bSetScrolling) ? 0 : DT_END_ELLIPSIS), 0);
			}

			SetTextColor(desthDC, m_clrFontColor);
		}

		drawRect.left = x + m_iLeftBorder+m_iRotationLeftBorder;
		drawRect.top = y + m_iTopBorder+m_iRotationTopBorder;
		drawRect.right = x + width - m_iRightBorder-m_iRotationRightBorder;
		drawRect.bottom = y + height - m_iBottomBorder-m_iRotationBottomBorder;

		DrawTextEx(desthDC, (char *) drawText.c_str(), drawText.length(), &drawRect, (bSetScrolling ? DT_LEFT : m_iFontAlign) | (bMultiLine ? DT_WORDBREAK : DT_SINGLELINE) | m_iFontVertAlign | ((m_bFontNoEllipsis || bSetScrolling) ? 0 : DT_END_ELLIPSIS), 0);

		Font = (HFONT) SelectObject(desthDC, tmpFont);
	}

	SetBkMode(desthDC, bkMode);
	SetTextColor(desthDC, textColor);
}

void PaintText::iapplyColorShadow(HDC desthDC, int x, int y, int width, int height, const string &drawText, bool bMultiLine, bool bSetScrolling)
{
	RECT drawRect;
	drawRect.left = x + m_iLeftBorder+m_iRotationLeftBorder;
	drawRect.top = y + m_iTopBorder+m_iRotationTopBorder;
	drawRect.right = x + width - m_iRightBorder-m_iRotationRightBorder;
	drawRect.bottom = y + height - m_iBottomBorder-m_iRotationBottomBorder;

	int bkMode = SetBkMode(desthDC, TRANSPARENT);
	int textColor = SetTextColor(desthDC, m_clrFontShadowColor);

	tmpFont = (HFONT) SelectObject(desthDC, Font);

	if ( m_iFontBlockedShadow != 0 )
	{
		for (int i = m_iFontBlockedShadow-1; i >= 0; i--)
		{
			OffsetRect(&drawRect, -1, 1);
			DrawTextEx(desthDC, (char *) drawText.c_str(), drawText.length(), &drawRect, (bSetScrolling ? DT_LEFT : m_iFontAlign) | (bMultiLine ? DT_WORDBREAK : DT_SINGLELINE) | m_iFontVertAlign | ((m_bFontNoEllipsis || bSetScrolling) ? 0 : DT_END_ELLIPSIS), 0);
		}
	}
	else
	{
		OffsetRect(&drawRect, m_iFontShadowX, m_iFontShadowY);
		DrawTextEx(desthDC, (char *) drawText.c_str(), drawText.length(), &drawRect, (bSetScrolling ? DT_LEFT : m_iFontAlign) | (bMultiLine ? DT_WORDBREAK : DT_SINGLELINE) | m_iFontVertAlign | ((m_bFontNoEllipsis || bSetScrolling) ? 0 : DT_END_ELLIPSIS), 0);
	}

	Font = (HFONT) SelectObject(desthDC, tmpFont);
	
	SetBkMode(desthDC, bkMode);
	SetTextColor(desthDC, textColor);
}
	
void PaintText::iapplyColorOutline(HDC desthDC, int x, int y, int width, int height, const string &drawText, bool bMultiLine, bool bSetScrolling)
{
	RECT drawRect;
	drawRect.left = x + m_iLeftBorder+m_iRotationLeftBorder;
	drawRect.top = y + m_iTopBorder+m_iRotationTopBorder;
	drawRect.right = x + width - m_iRightBorder-m_iRotationRightBorder;
	drawRect.bottom = y + height - m_iBottomBorder-m_iRotationBottomBorder;

	int bkMode = SetBkMode(desthDC, TRANSPARENT);
	int textColor = SetTextColor(desthDC, m_clrFontOutlineColor);

	tmpFont = (HFONT) SelectObject(desthDC, Font);

	if ( !m_bFontOutlineExtra )
	{
		OffsetRect(&drawRect, 1, 0);
		DrawTextEx(desthDC, (char *) drawText.c_str(), drawText.length(), &drawRect, (bSetScrolling ? DT_LEFT : m_iFontAlign) | (bMultiLine ? DT_WORDBREAK : DT_SINGLELINE) | m_iFontVertAlign | ((m_bFontNoEllipsis || bSetScrolling) ? 0 : DT_END_ELLIPSIS), 0);

		OffsetRect(&drawRect, -2, 0);
		DrawTextEx(desthDC, (char *) drawText.c_str(), drawText.length(), &drawRect, (bSetScrolling ? DT_LEFT : m_iFontAlign) | (bMultiLine ? DT_WORDBREAK : DT_SINGLELINE) | m_iFontVertAlign | ((m_bFontNoEllipsis || bSetScrolling) ? 0 : DT_END_ELLIPSIS), 0);

		OffsetRect(&drawRect, 1, -1);
		DrawTextEx(desthDC, (char *) drawText.c_str(), drawText.length(), &drawRect, (bSetScrolling ? DT_LEFT : m_iFontAlign) | (bMultiLine ? DT_WORDBREAK : DT_SINGLELINE) | m_iFontVertAlign | ((m_bFontNoEllipsis || bSetScrolling) ? 0 : DT_END_ELLIPSIS), 0);

		OffsetRect(&drawRect, 0, 2);
		DrawTextEx(desthDC, (char *) drawText.c_str(), drawText.length(), &drawRect, (bSetScrolling ? DT_LEFT : m_iFontAlign) | (bMultiLine ? DT_WORDBREAK : DT_SINGLELINE) | m_iFontVertAlign | ((m_bFontNoEllipsis || bSetScrolling) ? 0 : DT_END_ELLIPSIS), 0);
	}
	else
	{
		OffsetRect(&drawRect, 1, 1);
		DrawTextEx(desthDC, (char *) drawText.c_str(), drawText.length(), &drawRect, (bSetScrolling ? DT_LEFT : m_iFontAlign) | (bMultiLine ? DT_WORDBREAK : DT_SINGLELINE) | m_iFontVertAlign | ((m_bFontNoEllipsis || bSetScrolling) ? 0 : DT_END_ELLIPSIS), 0);

		OffsetRect(&drawRect, -1, 0);
		DrawTextEx(desthDC, (char *) drawText.c_str(), drawText.length(), &drawRect, (bSetScrolling ? DT_LEFT : m_iFontAlign) | (bMultiLine ? DT_WORDBREAK : DT_SINGLELINE) | m_iFontVertAlign | ((m_bFontNoEllipsis || bSetScrolling) ? 0 : DT_END_ELLIPSIS), 0);

		OffsetRect(&drawRect, -1, 0);
		DrawTextEx(desthDC, (char *) drawText.c_str(), drawText.length(), &drawRect, (bSetScrolling ? DT_LEFT : m_iFontAlign) | (bMultiLine ? DT_WORDBREAK : DT_SINGLELINE) | m_iFontVertAlign | ((m_bFontNoEllipsis || bSetScrolling) ? 0 : DT_END_ELLIPSIS), 0);

		OffsetRect(&drawRect, 0, -1);
		DrawTextEx(desthDC, (char *) drawText.c_str(), drawText.length(), &drawRect, (bSetScrolling ? DT_LEFT : m_iFontAlign) | (bMultiLine ? DT_WORDBREAK : DT_SINGLELINE) | m_iFontVertAlign | ((m_bFontNoEllipsis || bSetScrolling) ? 0 : DT_END_ELLIPSIS), 0);

		OffsetRect(&drawRect, 0, -1);
		DrawTextEx(desthDC, (char *) drawText.c_str(), drawText.length(), &drawRect, (bSetScrolling ? DT_LEFT : m_iFontAlign) | (bMultiLine ? DT_WORDBREAK : DT_SINGLELINE) | m_iFontVertAlign | ((m_bFontNoEllipsis || bSetScrolling) ? 0 : DT_END_ELLIPSIS), 0);

		OffsetRect(&drawRect, 1, 0);
		DrawTextEx(desthDC, (char *) drawText.c_str(), drawText.length(), &drawRect, (bSetScrolling ? DT_LEFT : m_iFontAlign) | (bMultiLine ? DT_WORDBREAK : DT_SINGLELINE) | m_iFontVertAlign | ((m_bFontNoEllipsis || bSetScrolling) ? 0 : DT_END_ELLIPSIS), 0);

		OffsetRect(&drawRect, 1, 0);
		DrawTextEx(desthDC, (char *) drawText.c_str(), drawText.length(), &drawRect, (bSetScrolling ? DT_LEFT : m_iFontAlign) | (bMultiLine ? DT_WORDBREAK : DT_SINGLELINE) | m_iFontVertAlign | ((m_bFontNoEllipsis || bSetScrolling) ? 0 : DT_END_ELLIPSIS), 0);

		OffsetRect(&drawRect, 0, 1);
		DrawTextEx(desthDC, (char *) drawText.c_str(), drawText.length(), &drawRect, (bSetScrolling ? DT_LEFT : m_iFontAlign) | (bMultiLine ? DT_WORDBREAK : DT_SINGLELINE) | m_iFontVertAlign | ((m_bFontNoEllipsis || bSetScrolling) ? 0 : DT_END_ELLIPSIS), 0);
	}

	Font = (HFONT) SelectObject(desthDC, tmpFont);
	
	SetBkMode(desthDC, bkMode);
	SetTextColor(desthDC, textColor);
}
	
void PaintText::iapplyColor(HDC desthDC, int x, int y, int width, int height, const string &drawText, bool bMultiLine, bool bSetScrolling)
{
	RECT drawRect;
	drawRect.left = x + m_iLeftBorder+m_iRotationLeftBorder;
	drawRect.top = y + m_iTopBorder+m_iRotationTopBorder;
	drawRect.right = x + width - m_iRightBorder-m_iRotationRightBorder;
	drawRect.bottom = y + height - m_iBottomBorder-m_iRotationBottomBorder;

	int bkMode = SetBkMode(desthDC, TRANSPARENT);
	int textColor = SetTextColor(desthDC, m_clrFontColor);

	tmpFont = (HFONT) SelectObject(desthDC, Font);

	DrawTextEx(desthDC, (char *) drawText.c_str(), drawText.length(), &drawRect, (bSetScrolling ? DT_LEFT : m_iFontAlign) | (bMultiLine ? DT_WORDBREAK : DT_SINGLELINE) | m_iFontVertAlign | ((m_bFontNoEllipsis || bSetScrolling) ? 0 : DT_END_ELLIPSIS), 0);

	Font = (HFONT) SelectObject(desthDC, tmpFont);

	SetBkMode(desthDC, bkMode);
	SetTextColor(desthDC, textColor);
}


POINT PaintText::measure(HDC desthDC, int x, int width, LPCSTR pszDrawText, BOOL bWithBorders)
{
	string drawText = pszDrawText;

	if (!bWithBorders)
	{
		return MeasureTextSize(desthDC, Font, drawText);
	}
	else
	{
		POINT returnValues = MeasureTextSize(desthDC, Font, drawText);

		returnValues.x = returnValues.x+m_iLeftBorder+m_iRightBorder+m_iRotationLeftBorder+m_iRotationRightBorder;
		returnValues.y = returnValues.y+m_iTopBorder+m_iBottomBorder+m_iRotationTopBorder+m_iRotationBottomBorder;

		return returnValues;
	}
}