/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2007 The LS-Universe Team

  Homepage: http://www.ls-universe.info

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "stdafx.h"
#include "PaintTooltip.h"

#include "PaintHelpers.h"
#include "SettingsHelpers.h"

#include "shlobj.h"
#include "windowsx.h"

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

PaintTooltip::PaintTooltip(HWND hWndParent, HINSTANCE hInstanceParent) :
	Font(NULL),
	
	hWndParentWindow(NULL),
	hInstanceParentWindow(NULL),
	hWndTooltip(NULL)
{
	hWndParentWindow = hWndParent;
	hInstanceParentWindow = hInstanceParent;
}

PaintTooltip::~PaintTooltip()
{
	if (Font)
	{
		DeleteObject(Font);
		Font = NULL;
	}

	RECT emptyRect = {0,0,0,0};
	
	TOOLINFO ti;
	ti.cbSize = sizeof(TOOLINFO);
	ti.uFlags = 0;
	ti.hwnd = hWndParentWindow;
	ti.uId = 0;
	ti.rect = emptyRect;
	ti.hinst = hInstanceParentWindow;
	ti.lpszText = NULL;
	ti.lParam = 0;

	SendMessage(hWndTooltip, TTM_DELTOOL, 0, (LPARAM) (LPTOOLINFO) &ti);

	DestroyWindow(hWndTooltip);
	hWndTooltip = NULL;
}

HWND PaintTooltip::configure(LPCSTR pszName, LPCSTR pszPrefix)
{
	string name(pszName);
	string prefix(pszPrefix);

	string strAddToGroup = iGetxRCString(name, (prefix + "TooltipAddToGroup"), "");

	if (!strAddToGroup.empty() && _stricmp(name.c_str(), strAddToGroup.c_str()) != NULL)
	{	
		return configure(strAddToGroup.c_str(), "");
	}

	//Initialize Basic Helper Strings
	int numTokens = 0;

	string one = "";
	string two = "";
	string three = "";
	string four = "";
	string five = "";
	string six = "";
	string seven = "";

	//Balloon ToolTips Initialization
	bool bBalloonTooltip = false;
	string strBalloonTitle = "";
	int iBalloonIcon = 0;

	numTokens = SplitString(iGetx2RCLine(name, (prefix + "TooltipBalloon"), "false"), one, two, three, four, five, six, seven);

	if ( _stricmp( one.c_str(), "false") != NULL && (IsOS(OS_2KXP) || IsOS(OS_ME)) )
	{
		bBalloonTooltip = true;

		if ( numTokens >= 1 && !one.empty() && _stricmp( one.c_str(), "true") != NULL )
		{
			strBalloonTitle = one;
			if ( numTokens >= 2 )
				iBalloonIcon = atoi( two.c_str() );
			if ( iBalloonIcon > 3 || iBalloonIcon < 0 )
				iBalloonIcon = 0;
		}
	}

	// struct specifying control classes to register
	INITCOMMONCONTROLSEX iccex; 

	/* INITIALIZE COMMON CONTROLS */
	iccex.dwICC = ICC_WIN95_CLASSES;
	iccex.dwSize = sizeof(INITCOMMONCONTROLSEX);

	InitCommonControlsEx(&iccex);

	hWndTooltip = CreateWindowEx(WS_EX_TOOLWINDOW | WS_EX_TOPMOST,
										TOOLTIPS_CLASS,
										TEXT(""),
										WS_POPUP | TTS_NOPREFIX | TTS_ALWAYSTIP | (bBalloonTooltip ? TTS_BALLOON : 0),
										CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
										NULL,
										NULL,
										hInstanceParentWindow,
										NULL);
	
	//Use WS_EX_TOPMOST in CreateWindowEx instead 
	//SetWindowPos(hWndTooltip, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE | SWP_NOOWNERZORDER);  

	//Destroy Font, if we have a Refresh
	if (Font)
	{
		DeleteObject(Font);
		Font = NULL;
	}
	
	const NameValuePair CharSetConstants[] =
	{
		{ "ANSI_CHARSET", ANSI_CHARSET },
		{ "BALTIC_CHARSET", BALTIC_CHARSET },
		{ "CHINESEBIG5_CHARSET", CHINESEBIG5_CHARSET },
		{ "DEFAULT_CHARSET", DEFAULT_CHARSET },
		{ "EASTEUROPE_CHARSET", EASTEUROPE_CHARSET },
		{ "GB2312_CHARSET", GB2312_CHARSET },
		{ "GREEK_CHARSET", GREEK_CHARSET },
		{ "HANGUL_CHARSET", HANGUL_CHARSET },
		{ "MAC_CHARSET", MAC_CHARSET },
		{ "RUSSIAN_CHARSET", RUSSIAN_CHARSET },
		{ "SHIFTJIS_CHARSET", SHIFTJIS_CHARSET },
		{ "SYMBOL_CHARSET", SYMBOL_CHARSET },
		{ "TURKISH_CHARSET", TURKISH_CHARSET },
		{ "VIETNAMESE_CHARSET", VIETNAMESE_CHARSET },
		{ "JOHAB_CHARSET", JOHAB_CHARSET },
		{ "ARABIC_CHARSET", ARABIC_CHARSET },
		{ "HEBREW_CHARSET", HEBREW_CHARSET },
		{ "THAI_CHARSET", THAI_CHARSET },
		{ "OEM_CHARSET", OEM_CHARSET },
		{ 0, 0 }
	};

	//Basic Font Settings
	string strFontName = iGetx2RCString(name, (prefix + "TooltipFont"), "Arial");
	int iFontHeight = iGetx2RCInt(name, (prefix + "TooltipFontHeight"), 15, 0);

	short shFontCharSet = iGetx2RCNamedValue(name, (prefix + "TooltipFontCharSet"), CharSetConstants, DEFAULT_CHARSET);

	bool bFontBold = iGetx2RCBool(name, (prefix + "TooltipFontBold"));
	bool bFontItalic = iGetx2RCBool(name, (prefix + "TooltipFontItalic"));
	bool bFontUnderline = iGetx2RCBool(name, (prefix + "TooltipFontUnderline"));

	bool bFontSmoothing = iGetx2RCBool(name, (prefix + "TooltipFontSmoothing"), true);
	bool bFontClearType = iGetx2RCBool(name, (prefix + "TooltipFontClearType"), true);

	CreateFontHandle(Font, strFontName, iFontHeight, bFontItalic, bFontUnderline, bFontBold, shFontCharSet, bFontSmoothing, bFontClearType);
		
	if ( Font )
		SetWindowFont(hWndTooltip, Font, FALSE);

	SendMessage(hWndTooltip, (UINT) TTM_SETTIPTEXTCOLOR, (WPARAM) (COLORREF) iGetx2RCColor(name, (prefix + "TooltipFontColor"), iGetx2RCColor(name, (prefix + "TooltipTextColor"), GetSysColor(COLOR_INFOTEXT))), 0);  
	SendMessage(hWndTooltip, (UINT) TTM_SETTIPBKCOLOR, (WPARAM) (COLORREF) iGetx2RCColor(name, (prefix + "TooltipColor"), GetSysColor(COLOR_INFOBK)), 0 );   

	// Balloon Title
	if ( bBalloonTooltip && !strBalloonTitle.empty() )
		SendMessage(hWndTooltip, TTM_SETTITLE, (WPARAM) (int) iBalloonIcon, (LPARAM) strBalloonTitle.c_str() );

	//Tooltip Width / AutoWidth
	int iMaxWidth = iGetx2RCInt(name, (prefix + "ToolTipMaxWidth"), 0, 0);

	if ( iMaxWidth != 0 )
		SendMessage(hWndTooltip, (UINT) TTM_SETMAXTIPWIDTH, (WPARAM) 0, (LPARAM) MAKELONG(iMaxWidth, 0));
	else
		SendMessage(hWndTooltip, (UINT) TTM_SETMAXTIPWIDTH, (WPARAM) 0, (LPARAM) MAKELONG(GetSystemMetrics(SM_CXSCREEN), 0));

	//Tooltip Durations
	CStr strToolDurations = iGetx2RCLine(name, (prefix + "ToolTipDurations"), "-1 -1").c_str();
	strToolDurations.Replace("\"", "");

	string strWork = strToolDurations;
	
	numTokens = SplitString(strWork, one , two, three, four, five, six, seven);
			
	int iInitialDuration = -1;
	int iAutoPopDuration = -1;

	if ( numTokens >= 1 )
	{
		iInitialDuration = atoi(one.c_str());
		if ( numTokens >= 2 )
			iAutoPopDuration = atoi(two.c_str());
	}

	if ( iInitialDuration < 0 )
		iInitialDuration = -1;
	if ( iAutoPopDuration != -1 && iAutoPopDuration < 1000 )
		iAutoPopDuration = 1000;

	if ( iInitialDuration != -1 )
		SendMessage(hWndTooltip, (UINT) TTM_SETDELAYTIME, (WPARAM) (DWORD) TTDT_INITIAL, (LPARAM) MAKELONG(iInitialDuration, 0) );
	if ( iAutoPopDuration != -1 )
		SendMessage(hWndTooltip, (UINT) TTM_SETDELAYTIME, (WPARAM) (DWORD) TTDT_AUTOPOP, (LPARAM) MAKELONG(iAutoPopDuration, 0) );

	//Finishing Initialization
	RECT rectParent;
	GetClientRect(hWndParentWindow, &rectParent);

	TOOLINFO ti;
	ti.cbSize = sizeof(TOOLINFO);
	ti.uFlags = TTF_TRANSPARENT;
	ti.hwnd = hWndParentWindow;
	ti.uId = 0;
	ti.rect = rectParent;
	ti.hinst = hInstanceParentWindow;
	ti.lpszText = "";
	ti.lParam = 0;

	SendMessage(hWndTooltip, TTM_ADDTOOL, 0, (LPARAM) (LPTOOLINFO) &ti);

	return hWndTooltip;
}
	
void PaintTooltip::apply(LPCSTR strTooltipText, BOOL bUpdateSize)
{
	string strProcessTooltip = strTooltipText;

	if ( !strProcessTooltip.empty() )
	{
		while ( strstr(strProcessTooltip.c_str(), "<br>") != NULL )
			strProcessTooltip.replace(strProcessTooltip.find("<br>"), 4, "\n", 1);

		while ( strstr(strProcessTooltip.c_str(), "<tab>") != NULL )
			strProcessTooltip.replace(strProcessTooltip.find("<tab>"), 5, "\t", 1);
		
		//Problematic if a "normal" \n or \t is in the tooltiptext
		//-> let the module coders do the work :P

		/*while ( strstr(strProcessTooltip.c_str(), "\\n") != NULL )
			strProcessTooltip.replace(strProcessTooltip.find("\\n"), 2, "\n", 1);
		
		while ( strstr(strProcessTooltip.c_str(), "\\t") != NULL )
			strProcessTooltip.replace(strProcessTooltip.find("\\t"), 2, "\t", 1);*/
	}

	char charTooltipText[MAX_LINE_LENGTH] = {0};
	strcpy(charTooltipText, strProcessTooltip.c_str());

	RECT rectParent;
	GetClientRect(hWndParentWindow, &rectParent);

	TOOLINFO ti;
	ti.cbSize = sizeof(TOOLINFO);
	ti.uFlags = TTF_TRANSPARENT;
	ti.hwnd = hWndParentWindow;
	ti.uId = 0;
	ti.rect = rectParent;
	ti.hinst = hInstanceParentWindow;
	ti.lpszText = charTooltipText;
	ti.lParam = 0;

	if (bUpdateSize)
	{
		SendMessage(hWndTooltip, TTM_NEWTOOLRECT, 0, (LPARAM) (LPTOOLINFO) &ti);
	}
	else
	{
		SendMessage(hWndTooltip, TTM_UPDATETIPTEXT, (WPARAM) 0, (LPARAM) (LPTOOLINFO) &ti);
	}
}
