/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2007 The LS-Universe Team

  Homepage: http://www.ls-universe.info

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "stdafx.h"
#include "SettingsHelpers.h"

#include "MultiMonitor.h"

extern MultiMonitor multimonitor;

//////////////////////////////////////////////////////////////////////
// DLL Internal
//////////////////////////////////////////////////////////////////////

void ProcessColor(const string &cfg, COLORREF &clrResult)
{
	string tmpColor = cfg;

	if ( strstr( tmpColor.c_str(), "#" ) != NULL )
	{
		while ( strstr( tmpColor.c_str(), "#" ) != NULL )
			tmpColor.replace(tmpColor.find("#"), 1, "", 0);
	}
	
	char szFirst[MAX_LINE_LENGTH] = {0};
	char szSecond[MAX_LINE_LENGTH] = {0};
	char szThird[MAX_LINE_LENGTH] = {0};

	LPSTR lpszTokens[3] = { szFirst, szSecond, szThird	};

	int nCount = LCTokenize(tmpColor.c_str(), lpszTokens, 3, NULL);

	if (nCount >= 3)
	{
		int nRed, nGreen, nBlue;

		nRed = strtol(szFirst, NULL, 10);
		nGreen = strtol(szSecond, NULL, 10);
		nBlue = strtol(szThird, NULL, 10);

		clrResult = RGB(nRed, nGreen, nBlue);
	}
	else if (nCount >= 1)
	{
		tmpColor = szFirst;

		if (tmpColor.length() == 3)
		{
			clrResult = strtol((tmpColor.substr(0,1) + tmpColor.substr(0,1) + tmpColor.substr(1,1) + tmpColor.substr(1,1) + tmpColor.substr(2,1) + tmpColor.substr(2,1)).c_str(), NULL, 16);
			// convert from BGR to RGB
			clrResult = RGB(GetBValue(clrResult), GetGValue(clrResult), GetRValue(clrResult));
		}
		else
		{
			clrResult = strtol(szFirst, NULL, 16);
            // convert from BGR to RGB
            clrResult = RGB(GetBValue(clrResult), GetGValue(clrResult), GetRValue(clrResult));
		}
	}
}

void ParseBGBevelColorCFG(const string &cfg, COLORREF &bgcolor, COLORREF &lightcolor, COLORREF &darkcolor)
{
	if ( cfg.empty() )
		return;

	CStr strProcessing = cfg.c_str();
	strProcessing.Replace("\"", "");

	char bg[32] = {0};
	char light[32] = {0};
	char dark[32] = {0};
	char *buffers[] = {bg, light, dark};

	int numTokens = LCTokenize(strProcessing, buffers, 3, NULL);

	if(numTokens == 1)
	{
		ProcessColor(bg, bgcolor);
		lightcolor = GetSysColor(COLOR_3DLIGHT);
		darkcolor = GetSysColor(COLOR_3DSHADOW);
	}
	else if (numTokens == 2)
	{
		ProcessColor(bg, bgcolor);
	
		ProcessColor(light, lightcolor);
		darkcolor = lightcolor;
	}
	else if (numTokens == 3)
	{
		ProcessColor(bg, bgcolor);

		ProcessColor(light, lightcolor);

		ProcessColor(dark, darkcolor);
	}
}

bool Parse4SizesCFG(const string &cfg, int &leftSize, int &rightSize, int &topSize, int &bottomSize, bool bOnlyPositiveValues)
{
	leftSize = 0;
	rightSize = 0;
	topSize = 0;
	bottomSize = 0;

	if ( cfg.empty() )
		return false;

	CStr strProcessing = cfg.c_str();
	strProcessing.Replace("\"", "");
	
	char left[32] = {0};
	char right[32] = {0};
	char top[32] = {0};
	char bottom[32] = {0};
	char *buffers[] = {left, right, top, bottom };

	int numTokens = LCTokenize(strProcessing, buffers, 4, NULL);
		
	if(numTokens == 1)
	{
		leftSize = rightSize = topSize = bottomSize = atoi(left);
	}
	else if (numTokens == 2)
	{
		leftSize = rightSize = atoi(left);
		topSize = bottomSize = atoi(right);
	}
	else if (numTokens == 4)
	{
		leftSize = atoi(left);
		rightSize = atoi(right);
		topSize = atoi(top);
		bottomSize = atoi(bottom);
	}

	if (bOnlyPositiveValues)
	{
		if ( leftSize <= 0 )
			leftSize = 0;

		if ( rightSize <= 0 )
			rightSize = 0;

		if ( topSize <= 0 )
			topSize = 0;

		if ( bottomSize <= 0 )
			bottomSize = 0;

		if ( leftSize > 0 || rightSize > 0 || topSize > 0 || bottomSize > 0 )
			return true;
		else
			return false;
	}
	else
	{
		if ( leftSize > 0 || rightSize > 0 || topSize > 0 || bottomSize > 0 )
			return true;
		else
			return false;
	}
}

void ParseRotationCFG(const string &cfg, int &degree, int &width, int &height)
{
	degree = 0;
	width = -1;
	height = -1;

	if ( cfg.empty() )
		return;

	CStr strProcessing = cfg.c_str();
	strProcessing.Replace("\"", "");
	
	char d[32] = {0};
	char w[32] = {0};
	char h[32] = {0};
	char *buffers[] = {d, w, h};

	int numTokens = LCTokenize(strProcessing, buffers, 3, NULL);
		
	if (numTokens == 1)
	{
		degree = atoi(d);
	}
	else if (numTokens == 3)
	{
		degree = atoi(d);
		width = atoi(w);
		height = atoi(h);
	}

	if ( degree <= -360 || degree >= 360 )
		degree = 0;

	if ( width <= 0 )
		width = -1;
	if ( height <= 0 )
		height = -1;
}

// trim leading and trailing whitespace
string trim(const string &toTrim)
{
	int start = 0;
	int end = toTrim.length() - 1;
	
	while(start <= end && isspace(toTrim[start]))
		start++;
	
	while(start <= end && isspace(toTrim[end]))
		end--;
	
	return toTrim.substr(start, end - start + 1);
}

string IntToString(int input)
{
	char buffer[32] = {0};
	itoa( input, buffer, 10 );
	return (string) buffer;
}

// retrieve a boolean value from the step.rc
bool iGetxRCBool(const string &uniqueKey, const string &baseName, bool defaultVal)
{
	string key = uniqueKey + baseName;

	return (GetRCBoolDef(key.c_str(), defaultVal) ? true : false);
}

// retrieve a boolean value from the step.rc
bool iGetx2RCBool(const string &uniqueKey, const string &baseName, bool defaultVal)
{	
	string strTmpValue = iGetxRCString(uniqueKey, baseName, "");
	//Special Setup Due to FALSE set BOOL value
	if ( _stricmp(strTmpValue.c_str(), "false") == NULL || _stricmp(strTmpValue.c_str(), "off") == NULL || _stricmp(strTmpValue.c_str(), "no") == NULL )
		return false;
	else if ( iGetxRCBool(uniqueKey, baseName) )
		return true;

	string strGroups = iGetxRCLine(uniqueKey, "AddToGroup", "");
	string strNextGroup = "";

	if ( !strGroups.empty() )
	{
		char szBuffer[MAX_LINE_LENGTH] = {0};
		LPCSTR pszNextToken = strGroups.c_str();

		while ( GetToken(pszNextToken, szBuffer, &pszNextToken, false) )
		{
			if ( strNextGroup.empty() )
				strNextGroup = szBuffer;

			if ( iGetxRCBool(szBuffer, baseName) )
				return iGetxRCBool(szBuffer, baseName, defaultVal);
		}

		if ( _stricmp(uniqueKey.c_str(), strNextGroup.c_str()) != NULL ) 
			return iGetx2RCBool(strNextGroup, baseName, defaultVal);
		else
			return iGetxRCBool(strNextGroup, baseName, iGetxRCBool("Global", baseName, defaultVal));
	}
	else
	{
		return iGetxRCBool(uniqueKey, baseName, iGetxRCBool("Global", baseName, defaultVal));
	}
}

// retrieve a string value from step.rc
string iGetxRCString(const string &uniqueKey, const string &baseName, const string &defaultVal)
{
	string key = uniqueKey + baseName;

	char buffer[MAX_LINE_LENGTH] = {0};
	GetRCString(key.c_str(), buffer, defaultVal.c_str(), MAX_LINE_LENGTH);

	return (string) buffer;
}

// retrieve a string value from step.rc
string iGetx2RCString(const string &uniqueKey, const string &baseName, const string &defaultVal)
{
	if ( iGetxRCBool(uniqueKey, baseName) )
		return iGetxRCString(uniqueKey, baseName, defaultVal);

	string strGroups = iGetxRCLine(uniqueKey, "AddToGroup", "");
	string strNextGroup = "";

	if ( !strGroups.empty() )
	{
		char szBuffer[MAX_LINE_LENGTH] = {0};
		LPCSTR pszNextToken = strGroups.c_str();

		while ( GetToken(pszNextToken, szBuffer, &pszNextToken, false) )
		{
			if ( strNextGroup.empty() )
				strNextGroup = szBuffer;

			if ( iGetxRCBool(szBuffer, baseName) )
				return iGetxRCString(szBuffer, baseName, defaultVal);
		}

		if ( _stricmp(uniqueKey.c_str(), strNextGroup.c_str()) != NULL )
			return iGetx2RCString(strNextGroup, baseName, defaultVal);
		else
			return iGetxRCString(strNextGroup, baseName, iGetxRCString("Global", baseName, defaultVal));
	}
	else
	{
		return iGetxRCString(uniqueKey, baseName, iGetxRCString("Global", baseName, defaultVal));
	}
}

// retrieve a line from step.rc
string iGetxRCLine(const string &uniqueKey, const string &baseName, const string &defaultVal)
{
	string key = uniqueKey + baseName;

	char buffer[MAX_LINE_LENGTH] = {0};
	GetRCLine(key.c_str(), buffer, MAX_LINE_LENGTH, defaultVal.c_str());

	return (string) buffer;
}

// retrieve a line from step.rc
string iGetx2RCLine(const string &uniqueKey, const string &baseName, const string &defaultVal)
{
	if ( iGetxRCBool(uniqueKey, baseName) )
		return iGetxRCLine(uniqueKey, baseName, defaultVal);

	string strGroups = iGetxRCLine(uniqueKey, "AddToGroup", "");
	string strNextGroup = "";

	if ( !strGroups.empty() )
	{
		char szBuffer[MAX_LINE_LENGTH] = {0};
		LPCSTR pszNextToken = strGroups.c_str();

		while ( GetToken(pszNextToken, szBuffer, &pszNextToken, false) )
		{
			if ( strNextGroup.empty() )
				strNextGroup = szBuffer;

			if ( iGetxRCBool(szBuffer, baseName) )
				return iGetxRCLine(szBuffer, baseName, defaultVal);
		}

		if ( _stricmp(uniqueKey.c_str(), strNextGroup.c_str()) != NULL )
			return iGetx2RCLine(strNextGroup, baseName, defaultVal);
		else
			return iGetxRCLine(strNextGroup, baseName, iGetxRCLine("Global", baseName, defaultVal));
	}
	else
	{
		return iGetxRCLine(uniqueKey, baseName, iGetxRCLine("Global", baseName, defaultVal));
	}
}

// retrieve a named value from step.rc
int iGetxRCNamedValue(const string &uniqueKey, const string &baseName, const NameValuePair *nameValuePairs, int defaultVal)
{
	string name = iGetxRCString(uniqueKey, baseName, "");

	const NameValuePair *current = nameValuePairs;

	if(name.length() > 0)
	{
		while(current->name)
		{
			if(_stricmp(current->name, name.c_str()) == 0)
				return current->value;

			current++;
		}
	}

	return defaultVal;
}

// retrieve a named value from step.rc
int iGetx2RCNamedValue(const string &uniqueKey, const string &baseName, const NameValuePair *nameValuePairs, int defaultVal)
{
	if ( iGetxRCBool(uniqueKey, baseName) )
		return iGetxRCNamedValue(uniqueKey, baseName, nameValuePairs, defaultVal);

	string strGroups = iGetxRCLine(uniqueKey, "AddToGroup", "");
	string strNextGroup = "";

	if ( !strGroups.empty() )
	{
		char szBuffer[MAX_LINE_LENGTH] = {0};
		LPCSTR pszNextToken = strGroups.c_str();

		while ( GetToken(pszNextToken, szBuffer, &pszNextToken, false) )
		{
			if ( strNextGroup.empty() )
				strNextGroup = szBuffer;

			if ( iGetxRCBool(szBuffer, baseName) )
				return iGetxRCNamedValue(szBuffer, baseName, nameValuePairs, defaultVal);
		}

		if ( _stricmp(uniqueKey.c_str(), strNextGroup.c_str()) != NULL )
			return iGetx2RCNamedValue(strNextGroup, baseName, nameValuePairs, defaultVal);
		else
			return iGetxRCNamedValue(strNextGroup, baseName, nameValuePairs, iGetxRCNamedValue("Global", baseName, nameValuePairs, defaultVal));
	}
	else
	{
		return iGetxRCNamedValue(uniqueKey, baseName, nameValuePairs, iGetxRCNamedValue("Global", baseName, nameValuePairs, defaultVal));
	}
}

// retrieve a color value from step.rc
int iGetxRCColor(const string &uniqueKey, const string &baseName, int defaultVal)
{
	string key = uniqueKey + baseName;

	COLORREF crReturn = (COLORREF) defaultVal;

	char buffer[MAX_LINE_LENGTH] = {0};
	GetRCLine(key.c_str(), buffer, MAX_LINE_LENGTH, "");
	string tmpColor = buffer;

	if (tmpColor.empty())
		return crReturn;

	ProcessColor(tmpColor, crReturn);
	
	return crReturn;
}

// retrieve a color value from step.rc
int iGetx2RCColor(const string &uniqueKey, const string &baseName, int defaultVal)
{
	if ( iGetxRCBool(uniqueKey, baseName) )
		return iGetxRCColor(uniqueKey, baseName, defaultVal);

	string strGroups = iGetxRCLine(uniqueKey, "AddToGroup", "");
	string strNextGroup = "";

	if ( !strGroups.empty() )
	{
		char szBuffer[MAX_LINE_LENGTH] = {0};
		LPCSTR pszNextToken = strGroups.c_str();

		while ( GetToken(pszNextToken, szBuffer, &pszNextToken, false) )
		{
			if ( strNextGroup.empty() )
				strNextGroup = szBuffer;

			if ( iGetxRCBool(szBuffer, baseName) )
				return iGetxRCColor(szBuffer, baseName, defaultVal);
		}

		if ( _stricmp(uniqueKey.c_str(), strNextGroup.c_str()) != NULL )
			return iGetx2RCColor(strNextGroup, baseName, defaultVal);
		else
			return iGetxRCColor(strNextGroup, baseName, iGetxRCColor("Global", baseName, defaultVal));
	}
	else
	{
		return iGetxRCColor(uniqueKey, baseName, iGetxRCColor("Global", baseName, defaultVal));
	}
}

// retrieve an integer value from step.rc, make sure it is between minVal and maxVal inclusive
int iGetxRCInt(const string &uniqueKey, const string &baseName, int defaultVal, int minVal, int maxVal)
{
	string key = uniqueKey + baseName;
	int answer = GetRCInt(key.c_str(), defaultVal);
	return max(min(answer, maxVal), minVal);
}

// retrieve an integer value from step.rc, make sure it is between minVal and maxVal inclusive
int iGetx2RCInt(const string &uniqueKey, const string &baseName, int defaultVal, int minVal, int maxVal)
{
	if ( iGetxRCBool(uniqueKey, baseName) )
		return iGetxRCInt(uniqueKey, baseName, defaultVal, minVal, maxVal);

	string strGroups = iGetxRCLine(uniqueKey, "AddToGroup", "");
	string strNextGroup = "";

	if ( !strGroups.empty() )
	{
		char szBuffer[MAX_LINE_LENGTH] = {0};
		LPCSTR pszNextToken = strGroups.c_str();

		while ( GetToken(pszNextToken, szBuffer, &pszNextToken, false) )
		{
			if ( strNextGroup.empty() )
				strNextGroup = szBuffer;

			if ( iGetxRCBool(szBuffer, baseName) )
				return iGetxRCInt(szBuffer, baseName, defaultVal, minVal, maxVal);
		}

		if ( _stricmp(uniqueKey.c_str(), strNextGroup.c_str()) != NULL )
			return iGetx2RCInt(strNextGroup, baseName, defaultVal, minVal, maxVal);
		else
			return iGetxRCInt(strNextGroup, baseName, iGetxRCInt("Global", baseName, defaultVal, minVal, maxVal), minVal, maxVal);
	}
	else
	{
		return iGetxRCInt(uniqueKey, baseName, iGetxRCInt("Global", baseName, defaultVal, minVal, maxVal), minVal, maxVal);
	}
}

// parse a coordinate specification
int ixParseCoord(const string &aString, int defaultVal, int maxVal, bool bYCoordinate)
{
	string strVal = trim(aString);

	if (strVal.length() == 0)
		return defaultVal;

	bool negative = false;
	bool percentage = false;
	bool extendedpercentage = false;
	bool realnegative = false;
	bool center = false;
	UINT i = 0;
	int value = 0;
	int extendedvalue = 0;

	if ( multimonitor.iRealMonitors > 1 && strstr(strVal.c_str(), "@") != NULL )
	{
		int minx = 0;
		int miny = 0;
		int maxwidth = maxVal;
		int maxheight = maxVal;
		if ( strstr(strVal.c_str(), "@") != NULL && (strVal[i] == 'v' || strVal[i] == 'V') && multimonitor.iRealMonitors >= 2 )
		{
			minx = multimonitor.iMinScreenX;
			miny = multimonitor.iMinScreenY;
			maxwidth = multimonitor.iMaxScreenWidth;
			maxheight = multimonitor.iMaxScreenHeight;
		}
		else if ( strstr(strVal.c_str(), "@") != NULL && strVal[i] == '2' && multimonitor.iRealMonitors >= 2 )
		{
			for (int x = 0; x < multimonitor.iMonitorIndex; x++)
			{
				if ( multimonitor.MonitorInfoArray[x].dwFlags != MONITORINFOF_PRIMARY )
				{
					minx = multimonitor.MonitorInfoArray[x].rcMonitor.left;
					miny = multimonitor.MonitorInfoArray[x].rcMonitor.top;
					maxwidth = multimonitor.MonitorSizeArray[x].cx;
					maxheight = multimonitor.MonitorSizeArray[x].cy;

					break;
				}
			}
		}
		else if ( strstr(strVal.c_str(), "@") != NULL && strVal[i] == '3' && multimonitor.iRealMonitors >= 3 )
		{
			int skip = 1;
			for (int x = 0; x < multimonitor.iMonitorIndex; x++)
			{
				if ( multimonitor.MonitorInfoArray[x].dwFlags != MONITORINFOF_PRIMARY )
				{
					if ( skip > 0 )
					{
						skip--;
						continue;
					}
					
					minx = multimonitor.MonitorInfoArray[x].rcMonitor.left;
					miny = multimonitor.MonitorInfoArray[x].rcMonitor.top;
					maxwidth = multimonitor.MonitorSizeArray[x].cx;
					maxheight = multimonitor.MonitorSizeArray[x].cy;

					break;
				}
			}
		}
		else
		{
			for (int x = 0; x < multimonitor.iMonitorIndex; x++)
			{
				if ( multimonitor.MonitorInfoArray[x].dwFlags == MONITORINFOF_PRIMARY )
				{
					minx = multimonitor.MonitorInfoArray[x].rcMonitor.left;
					miny = multimonitor.MonitorInfoArray[x].rcMonitor.top;
					maxwidth = multimonitor.MonitorSizeArray[x].cx;
					maxheight = multimonitor.MonitorSizeArray[x].cy;

					break;
				}
			}
		}
		
		if ( strstr(strVal.c_str(), "@") != NULL )
			i = i+2;

		if (strVal[i] == '-')
		{
			negative = true;
			i++;
		}
		else if (strVal[i] == '~')
		{
			realnegative = true;
			i++;
		}
		else if (strVal[i] == '+')
		{
			i++;
		}

		while (i < strVal.length())
		{
			if(strVal[i] >= '0' && strVal[i] <= '9')
			{
				value = (value * 10) + (strVal[i] - '0');
			}
			else
			{
				if(strVal[i] == 'c' || strVal[i] == 'C')
				{
					center = true;
				}
				else if (strVal[i] == '%')
				{
					percentage = true;

					if (strVal[i+1] == '+' )
					{
						extendedpercentage = true;
						i++;

						while (i < strVal.length())
						{
							if (strVal[i] >= '0' && strVal[i] <= '9')
							{
								extendedvalue = (extendedvalue * 10) + (strVal[i] - '0');
							}
							i++;
						}
					}
					else if (strVal[i+1] == '-' )
					{
						extendedpercentage = true;
						i++;

						while (i < strVal.length())
						{
							if (strVal[i] >= '0' && strVal[i] <= '9')
							{
								extendedvalue = (extendedvalue * 10) + (strVal[i] - '0');
							}
							i++;
						}
						extendedvalue = extendedvalue*(-1);
					}
				}
				break;
			}

			i++;
		}

		if (center)
		{
			if (realnegative)
				negative = true;

			if (negative)
				value = (bYCoordinate ? miny+(maxheight/2) : minx+(maxwidth/2)) - value;
			else
				value = (bYCoordinate ? miny+(maxheight/2) : minx+(maxwidth/2)) + value;
		}
		else if (realnegative)
		{
			value = (bYCoordinate ? miny : minx) - value;
		}
		else if (negative)
		{
			value = (bYCoordinate ? miny+maxheight : minx+maxwidth) - value;
		}
		else if	(percentage)
		{
			value = (value * (bYCoordinate ? maxheight : maxwidth)) / 100;
			if (extendedpercentage)
				value = value + extendedvalue;
		}
		else
		{
			value = (bYCoordinate ? miny : minx) + value;
		}
	}
	else
	{
		if ( strstr(strVal.c_str(), "@") != NULL )
			i = i+2;

		if (strVal[i] == '-')
		{
			negative = true;
			i++;
		}
		else if (strVal[i] == '~')
		{
			realnegative = true;
			i++;
		}
		else if (strVal[i] == '+')
		{
			i++;
		}

		while (i < strVal.length())
		{
			if(strVal[i] >= '0' && strVal[i] <= '9')
			{
				value = (value * 10) + (strVal[i] - '0');
			}
			else
			{
				if(strVal[i] == 'c' || strVal[i] == 'C')
				{
					center = true;
				}
				else if (strVal[i] == '%')
				{
					percentage = true;

					if (strVal[i+1] == '+' )
					{
						extendedpercentage = true;
						i++;

						while (i < strVal.length())
						{
							if (strVal[i] >= '0' && strVal[i] <= '9')
							{
								extendedvalue = (extendedvalue * 10) + (strVal[i] - '0');
							}
							i++;
						}
					}
					else if (strVal[i+1] == '-' )
					{
						extendedpercentage = true;
						i++;

						while (i < strVal.length())
						{
							if (strVal[i] >= '0' && strVal[i] <= '9')
							{
								extendedvalue = (extendedvalue * 10) + (strVal[i] - '0');
							}
							i++;
						}
						extendedvalue = extendedvalue*(-1);
					}
				}
				break;
			}

			i++;
		}

		if (center)
		{
			if (realnegative)
				negative = true;

			if (negative)
				value = (maxVal/2) - value;
			else
				value = (maxVal/2) + value;
		}
		else if (realnegative)
		{
			value = -value;
		}
		else if (negative)
		{
			value = maxVal - value;
		}
		else if	(percentage)
		{
			value = (value * maxVal) / 100;
			if (extendedpercentage)
				value = value + extendedvalue;
		}
	}
	
	return value;
}

// retrieve a coordinate from step.rc, accounting for negative values and centering
int iGetxRCCoord(const string &uniqueKey, const string &baseName, int defaultVal, int maxVal, bool bYCoordinate)
{
	string strVal = iGetxRCString(uniqueKey, baseName, "");
	return ixParseCoord(strVal, defaultVal, maxVal, bYCoordinate);
}

int iGetx2RCCoord(const string &uniqueKey, const string &baseName, int defaultVal, int maxVal, bool bYCoordinate)
{
	if ( iGetxRCBool(uniqueKey, baseName) )
		return iGetxRCCoord(uniqueKey, baseName, defaultVal, maxVal, bYCoordinate);

	string strGroups = iGetxRCLine(uniqueKey, "AddToGroup", "");
	string strNextGroup = "";

	if ( !strGroups.empty() )
	{
		char szBuffer[MAX_LINE_LENGTH] = {0};
		LPCSTR pszNextToken = strGroups.c_str();

		while ( GetToken(pszNextToken, szBuffer, &pszNextToken, false) )
		{
			if ( strNextGroup.empty() )
				strNextGroup = szBuffer;

			if ( iGetxRCBool(szBuffer, baseName) )
				return iGetxRCCoord(szBuffer, baseName, defaultVal, maxVal, bYCoordinate);
		}

		if ( _stricmp(uniqueKey.c_str(), strNextGroup.c_str()) != NULL )
			return iGetx2RCCoord(strNextGroup, baseName, defaultVal, maxVal, bYCoordinate);
		else
			return iGetxRCCoord(strNextGroup, baseName, iGetxRCCoord("Global", baseName, defaultVal, maxVal, bYCoordinate), maxVal, bYCoordinate);
	}
	else
	{
		return iGetxRCCoord(uniqueKey, baseName, iGetxRCCoord("Global", baseName, defaultVal, maxVal, bYCoordinate), maxVal, bYCoordinate);
	}
}

// parse a dimension
int ixParseDimen(const string &aString, int defaultVal, int maxVal, bool bYCoordinate)
{	
	string strVal = trim(aString);

	if (strVal.length() == 0)
		return defaultVal;

	bool negative = false;
	bool percentage = false;
	bool extendedpercentage = false;
	UINT i = 0;
	int value = 0;
	int extendedvalue = 0;

	if ( multimonitor.iRealMonitors > 1 && strstr(strVal.c_str(), "@") != NULL )
	{
		int maxwidth = maxVal;
		int maxheight = maxVal;
		if ( strstr(strVal.c_str(), "@") != NULL && (strVal[i] == 'v' || strVal[i] == 'V') && multimonitor.iRealMonitors >= 2 )
		{
			maxwidth = multimonitor.iMaxScreenWidth;
			maxheight = multimonitor.iMaxScreenHeight;
		}
		else if ( strstr(strVal.c_str(), "@") != NULL && strVal[i] == '2' && multimonitor.iRealMonitors >= 2 )
		{
			for (int x = 0; x < multimonitor.iMonitorIndex; x++)
			{
				if ( multimonitor.MonitorInfoArray[x].dwFlags != MONITORINFOF_PRIMARY )
				{
					maxwidth = multimonitor.MonitorSizeArray[x].cx;
					maxheight = multimonitor.MonitorSizeArray[x].cy;

					break;
				}
			}
		}
		else if ( strstr(strVal.c_str(), "@") != NULL && strVal[i] == '3' && multimonitor.iRealMonitors >= 3 )
		{
			int skip = 1;
			for (int x = 0; x < multimonitor.iMonitorIndex; x++)
			{
				if ( multimonitor.MonitorInfoArray[x].dwFlags != MONITORINFOF_PRIMARY )
				{
					if ( skip > 0 )
					{
						skip--;
						continue;
					}
			
					maxwidth = multimonitor.MonitorSizeArray[x].cx;
					maxheight = multimonitor.MonitorSizeArray[x].cy;

					break;
				}
			}
		}
		else
		{
			for (int x = 0; x < multimonitor.iMonitorIndex; x++)
			{
				if ( multimonitor.MonitorInfoArray[x].dwFlags == MONITORINFOF_PRIMARY )
				{
					maxwidth = multimonitor.MonitorSizeArray[x].cx;
					maxheight = multimonitor.MonitorSizeArray[x].cy;

					break;
				}
			}
		}

		if ( strstr(strVal.c_str(), "@") != NULL )
			i = i+2;

		if (strVal[i] == '-')
		{
			negative = true;
			i++;
		}

		while (i < strVal.length())
		{
			if (strVal[i] >= '0' && strVal[i] <= '9')
			{
				value = (value * 10) + (strVal[i] - '0');
			}
			else
			{
				if (strVal[i] == '%')
					percentage = true;

				if (strVal[i+1] == '+' )
				{
					extendedpercentage = true;
					i++;

					while (i < strVal.length())
					{
						if (strVal[i] >= '0' && strVal[i] <= '9')
						{
							extendedvalue = (extendedvalue * 10) + (strVal[i] - '0');
						}
						i++;
					}
				}
				else if (strVal[i+1] == '-' )
				{
					extendedpercentage = true;
					i++;

					while (i < strVal.length())
					{
						if (strVal[i] >= '0' && strVal[i] <= '9')
						{
							extendedvalue = (extendedvalue * 10) + (strVal[i] - '0');
						}
						i++;
					}
					extendedvalue = extendedvalue*(-1);
				}

				break;
			}

			i++;
		}

		if (percentage)
		{
			value = (value * (bYCoordinate ? maxheight : maxwidth)) / 100;
			if (extendedpercentage)
				value = value + extendedvalue;
		}

		if (negative)
			value = (bYCoordinate ? maxheight : maxwidth) - value;
	}
	else
	{
		if ( strstr(strVal.c_str(), "@") != NULL )
			i = i+2;

		if (strVal[i] == '-')
		{
			negative = true;
			i++;
		}

		while (i < strVal.length())
		{
			if (strVal[i] >= '0' && strVal[i] <= '9')
			{
				value = (value * 10) + (strVal[i] - '0');
			}
			else
			{
				if (strVal[i] == '%')
					percentage = true;

				if (strVal[i+1] == '+' )
				{
					extendedpercentage = true;
					i++;

					while (i < strVal.length())
					{
						if (strVal[i] >= '0' && strVal[i] <= '9')
						{
							extendedvalue = (extendedvalue * 10) + (strVal[i] - '0');
						}
						i++;
					}
				}
				else if (strVal[i+1] == '-' )
				{
					extendedpercentage = true;
					i++;

					while (i < strVal.length())
					{
						if (strVal[i] >= '0' && strVal[i] <= '9')
						{
							extendedvalue = (extendedvalue * 10) + (strVal[i] - '0');
						}
						i++;
					}
					extendedvalue = extendedvalue*(-1);
				}

				break;
			}

			i++;
		}

		if (percentage)
		{
			value = (value * maxVal) / 100;
			if (extendedpercentage)
				value = value + extendedvalue;
		}

		if (negative)
			value = maxVal - value;
	}
	
	return value;
}

// retrieve a dimension (absolute value or percentage) from step.rc
int iGetxRCDimen(const string &uniqueKey, const string &baseName, int defaultVal, int maxVal, bool bYCoordinate)
{
	string strVal = iGetxRCString(uniqueKey, baseName, "");

	if (_stricmp(strVal.c_str(), ".image") == NULL)
		return defaultVal;
	else
		return ixParseDimen(strVal, defaultVal, maxVal, bYCoordinate);
}

// retrieve a dimension (absolute value or percentage) from step.rc
int iGetx2RCDimen(const string &uniqueKey, const string &baseName, int defaultVal, int maxVal, bool bYCoordinate)
{
	if ( iGetxRCBool(uniqueKey, baseName) )
		return iGetxRCDimen(uniqueKey, baseName, defaultVal, maxVal, bYCoordinate);

	string strGroups = iGetxRCLine(uniqueKey, "AddToGroup", "");
	string strNextGroup = "";

	if ( !strGroups.empty() )
	{
		char szBuffer[MAX_LINE_LENGTH] = {0};
		LPCSTR pszNextToken = strGroups.c_str();

		while ( GetToken(pszNextToken, szBuffer, &pszNextToken, false) )
		{
			if ( strNextGroup.empty() )
				strNextGroup = szBuffer;

			if ( iGetxRCBool(szBuffer, baseName) )
				return iGetxRCDimen(szBuffer, baseName, defaultVal, maxVal, bYCoordinate);
		}

		if ( _stricmp(uniqueKey.c_str(), strNextGroup.c_str()) != NULL )
			return iGetx2RCDimen(strNextGroup, baseName, defaultVal, maxVal, bYCoordinate);
		else
			return iGetxRCDimen(strNextGroup, baseName, iGetxRCDimen("Global", baseName, defaultVal, maxVal, bYCoordinate), maxVal, bYCoordinate);
	}
	else
	{
		return iGetxRCDimen(uniqueKey, baseName, iGetxRCDimen("Global", baseName, defaultVal, maxVal, bYCoordinate), maxVal, bYCoordinate);
	}
}

int SplitString(const string &process, string &one, string &two, string &three, string &four, string &five, string &six, string &seven)
{
	int count = 0;

	one = "";
	two = "";
	three = "";
	four = "";
	five = "";
	six = "";
	seven = "";

	char cone[256] = {0};
	char ctwo[256] = {0};
	char cthree[256] = {0};
	char cfour[256] = {0};
	char cfive[256] = {0};
	char csix[256] = {0};
	char cseven[256] = {0};
	char *cbuffers[] = {cone, ctwo, cthree, cfour, cfive, csix, cseven};
		
	int numTokens = LCTokenize(process.c_str(), cbuffers, 7, NULL);
	
	if( numTokens >= 1 )
	{
		one = cone;
		count++;
		
		if ( numTokens >= 2 )
		{
			two = ctwo;
			count++;
		
			if ( numTokens >= 3 )
			{
				three = cthree;
				count++;
			
				if ( numTokens >= 4 )
				{
					four = cfour;
					count++;

					if ( numTokens >= 5 )
					{
						five = cfive;
						count++;

						if ( numTokens >= 6 )
						{
							six = csix;
							count++;
		
							if ( numTokens >= 7 )
							{
								seven = cseven;
								count++;
							}
						}
					}
				}
			}
		}
	}

	return count;
}

//////////////////////////////////////////////////////////////////////
// Exported in xExports.h
//////////////////////////////////////////////////////////////////////

BOOL GetxRCBool(LPCSTR uniqueKey, LPCSTR baseName, BOOL defaultVal)
{
	return (iGetxRCBool(uniqueKey, baseName, (defaultVal ? true : false)) ? TRUE : FALSE);
}

BOOL Getx2RCBool(LPCSTR uniqueKey, LPCSTR baseName, BOOL defaultVal)
{
	return (iGetx2RCBool(uniqueKey, baseName, (defaultVal ? true : false)) ? TRUE : FALSE);
}

void GetxRCString(LPSTR pszValue, LPCSTR uniqueKey, LPCSTR baseName, LPCSTR defaultVal)
{
	StringCchCopy(pszValue, 4096, (iGetxRCString(uniqueKey, baseName, defaultVal)).c_str()); 
}
void Getx2RCString(LPSTR pszValue, LPCSTR uniqueKey, LPCSTR baseName, LPCSTR defaultVal)
{
	StringCchCopy(pszValue, 4096, (iGetx2RCString(uniqueKey, baseName, defaultVal)).c_str());
}

void GetxRCLine(LPSTR pszValue, LPCSTR uniqueKey, LPCSTR baseName, LPCSTR defaultVal)
{
	StringCchCopy(pszValue, 4096, (iGetxRCLine(uniqueKey, baseName, defaultVal)).c_str());
}
void Getx2RCLine(LPSTR pszValue, LPCSTR uniqueKey, LPCSTR baseName, LPCSTR defaultVal)
{
	StringCchCopy(pszValue, 4096, (iGetx2RCLine(uniqueKey, baseName, defaultVal)).c_str());
}

int GetxRCNamedValue(LPCSTR uniqueKey, LPCSTR baseName, const NameValuePair *nameValuePairs, int defaultVal)
{
	return iGetxRCNamedValue(uniqueKey, baseName, nameValuePairs, defaultVal);
}
int Getx2RCNamedValue(LPCSTR uniqueKey, LPCSTR baseName, const NameValuePair *nameValuePairs, int defaultVal)
{
	return iGetx2RCNamedValue(uniqueKey, baseName, nameValuePairs, defaultVal);
}

int GetxRCColor(LPCSTR uniqueKey, LPCSTR baseName, int defaultVal)
{
	return iGetxRCColor(uniqueKey, baseName, defaultVal);
}
int Getx2RCColor(LPCSTR uniqueKey, LPCSTR baseName, int defaultVal)
{
	return iGetx2RCColor(uniqueKey, baseName, defaultVal);
}

int GetxRCInt(LPCSTR uniqueKey, LPCSTR baseName, int defaultVal, int minVal, int maxVal)
{
	return iGetxRCInt(uniqueKey, baseName, defaultVal, minVal, maxVal);
}
int Getx2RCInt(LPCSTR uniqueKey, LPCSTR baseName, int defaultVal, int minVal, int maxVal)
{
	return iGetx2RCInt(uniqueKey, baseName, defaultVal, minVal, maxVal);
}

int xParseCoord(LPCSTR aString, int defaultVal, int maxVal, BOOL bYCoordinate)
{
	return ixParseCoord(aString, defaultVal, maxVal, (bYCoordinate ? true : false));
}
int GetxRCCoord(LPCSTR uniqueKey, LPCSTR baseName, int defaultVal, int maxVal, BOOL bYCoordinate)
{
	return iGetxRCCoord(uniqueKey, baseName, defaultVal, maxVal, (bYCoordinate ? true : false));
}
int Getx2RCCoord(LPCSTR uniqueKey, LPCSTR baseName, int defaultVal, int maxVal, BOOL bYCoordinate)
{
	return iGetx2RCCoord(uniqueKey, baseName, defaultVal, maxVal, (bYCoordinate ? true : false));
}

int xParseDimen(LPCSTR aString, int defaultVal, int maxVal, BOOL bYCoordinate)
{
	return ixParseDimen(aString, defaultVal, maxVal, (bYCoordinate ? true : false));
}

int GetxRCDimen(LPCSTR uniqueKey, LPCSTR baseName, int defaultVal, int maxVal, BOOL bYCoordinate)
{
	return iGetxRCDimen(uniqueKey, baseName, defaultVal, maxVal, (bYCoordinate ? true : false));
}
int Getx2RCDimen(LPCSTR uniqueKey, LPCSTR baseName, int defaultVal, int maxVal, BOOL bYCoordinate)
{
	return iGetx2RCDimen(uniqueKey, baseName, defaultVal, maxVal, (bYCoordinate ? true : false));
}
